﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class AdminManager
    {
        private static List<AdminItem> items = new List<AdminItem>();
        private static List<InvalidLoginItem> il = new List<InvalidLoginItem>();

        public static void EnableAdmin(IUser userobj, bool enable)
        {
            if (enable)
            {
                if (!Settings.EnableAdmins)
                {
                    Settings.EnableAdmins = true;
                    String str = StringTemplate.GetString(STType.EnableDisable, 26);

                    if (str != null)
                        Callisto.PrintToHosts(str.Replace("+n", userobj.Name));

                    AdminLog.AddLine("room commands enabled by " + userobj.Name);
                }
            }
            else
            {
                if (Settings.EnableAdmins)
                {
                    Settings.EnableAdmins = false;
                    String str = StringTemplate.GetString(STType.EnableDisable, 27);

                    if (str != null)
                        Callisto.PrintToHosts(str.Replace("+n", userobj.Name));

                    AdminLog.AddLine("room commands disabled by " + userobj.Name);
                }
            }
        }

        public static void InvalidLoginAttempt(IUser userobj)
        {
            il.Add(new InvalidLoginItem(userobj));
            int i = il.FindAll(x => x.ip.Equals(userobj.ExternalIP) || x.guid == userobj.Guid).Count;

            if (i > 2)
            {
                ClearInvalidAttempts(userobj);
                String str = StringTemplate.GetString(STType.AdminLogin, 0);

                if (str != null)
                    Callisto.Print(str.Replace("+n", userobj.Name), true);

                AdminLog.AddLine(userobj.Name + " banned for invalid logins");
                BanManager.AddBanStatItem(new BanStatItem
                {
                    Name = userobj.Name,
                    IP = userobj.ExternalIP,
                    Admin = null,
                    Index = 4
                });
                userobj.Ban();
            }
            else
            {
                String str = StringTemplate.GetString(STType.AdminLogin, 1);

                if (str != null)
                    Callisto.Print(str.Replace("+n", userobj.Name), true);

                AdminLog.AddLine("invalid login attempt from " + userobj.Name);
            }
        }

        public static void ClearInvalidAttempts(IUser userobj)
        {
            il.RemoveAll(x => x.ip.Equals(userobj.ExternalIP) || x.guid == userobj.Guid);
        }

        public static void SetAdminLevel(IUser userobj)
        {
            AdminItem u = items.Find(x => (x.guid == userobj.Guid &&
                x.ip.GetAddressBytes()[0] == userobj.ExternalIP.GetAddressBytes()[0] &&
                x.ip.GetAddressBytes()[1] == userobj.ExternalIP.GetAddressBytes()[1]) ||
                x.ip.Equals(userobj.ExternalIP));

            if (u != null)
            {
                userobj.Level = u.level;
                ClearInvalidAttempts(userobj);
            }
        }

        public static void TempGrant(IUser adder, IUser userobj, int i)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : adder.Name;

            if (i > 3 || i < 1)
                return;

            if (userobj.IsLocalHost)
                return;

            userobj.Level = (AdminLevel)i;
            String str = StringTemplate.GetString(STType.AdminLogin, 2);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", userobj.Name).Replace("+x", ((int)userobj.Level).ToString()), true);

            AdminLog.AddLine(adder.Name + " gives " + userobj.Name + " temporary admin");
        }

        public static void TempUngrant(IUser adder, IUser userobj)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : adder.Name;

            if (userobj.IsLocalHost)
                return;

            userobj.Level = AdminLevel.Regular;
            String str = StringTemplate.GetString(STType.AdminLogin, 3);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", userobj.Name), true);

            AdminLog.AddLine(adder.Name + " removes " + userobj.Name + "'s admin level");
        }

        public static void AddAdmin(IUser adder, IUser userobj, int i)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : adder.Name;

            if (i > 3 || i < 1)
                return;

            String str = StringTemplate.GetString(STType.AdminLogin, 4);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", userobj.Name), true);

            AdminLog.AddLine(adder.Name + " has added " + userobj.Name + " to auto login");
            AdminLevel l = (AdminLevel)i;

            int u = items.FindIndex(x => (x.guid == userobj.Guid &&
                x.ip.GetAddressBytes()[0] == userobj.ExternalIP.GetAddressBytes()[0] &&
                x.ip.GetAddressBytes()[1] == userobj.ExternalIP.GetAddressBytes()[1]) ||
                x.ip.Equals(userobj.ExternalIP));

            if (u > -1)
            {
                items[u].level = l;
                SetAdminLevel(userobj);
                Save();
                return;
            }

            AdminItem a = new AdminItem();
            a.guid = userobj.Guid;
            a.ip = userobj.ExternalIP;
            a.level = l;
            a.name = userobj.Name;
            items.Add(a);
            SetAdminLevel(userobj);
            Save();
        }

        public static void RemoveAdmin(IUser remover, int i)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : remover.Name;

            if (i >= items.Count || i < 0)
                return;

            AdminItem a = items[i];
            String str = StringTemplate.GetString(STType.AdminLogin, 5);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", a.name), true);

            AdminLog.AddLine(remover.Name + " has removed " + a.name + " from auto login");
            items.RemoveAt(i);
            Save();

            for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
            {
                if (u.ExternalIP.Equals(a.ip))
                {
                    u.Level = AdminLevel.Regular;
                }
                else
                {
                    if (u.Guid == a.guid)
                        if (u.ExternalIP.GetAddressBytes()[0] == a.ip.GetAddressBytes()[0])
                            if (u.ExternalIP.GetAddressBytes()[1] == a.ip.GetAddressBytes()[1])
                                u.Level = AdminLevel.Regular;
                }
            }
        }

        public static void ListAdmin(IUser asker)
        {
            for (int i = 0; i < items.Count; i++)
                Callisto.Print(asker, i + " - " + items[i].name + " [" + items[i].ip + "] [" + items[i].level + "]");
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "admins.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("admins");

                while (xml.ReadToFollowing("item"))
                {
                    AdminItem i = new AdminItem();
                    xml.ReadSubtree().ReadToFollowing("name");
                    i.name = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("ip");
                    i.ip = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("guid");
                    i.guid = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("level");
                    i.level = (AdminLevel)xml.ReadElementContentAsInt();

                    if ((int)i.level > 3) i.level = AdminLevel.Host;
                    if ((int)i.level < 1) i.level = AdminLevel.Moderator;

                    items.Add(i);
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "admins.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("admins");

                foreach (AdminItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.name)));
                    xml.WriteElementString("ip", Convert.ToBase64String(a.ip.GetAddressBytes()));
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("level", ((int)a.level).ToString());
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class AdminItem
    {
        public String name;
        public IPAddress ip;
        public String guid;
        public AdminLevel level;
    }

    class InvalidLoginItem
    {
        public String guid;
        public IPAddress ip;

        public InvalidLoginItem(IUser userobj)
        {
            this.guid = userobj.Guid;
            this.ip = userobj.ExternalIP;
        }
    }
}
