﻿using System;
using System.Collections.Generic;
using System.Net;
using ICallisto;

namespace CallistoX
{
    class BanManager
    {
        private static List<int> bs = new List<int>();
        private static List<int> af = new List<int>();
        private static IPAddress last_bansend = IPAddress.Any;

        public static Rejections RejectionType = Rejections.Unknown;

        private static List<BanStatItem> bstats = new List<BanStatItem>();

        public static void AddBanStatItem(BanStatItem item)
        {
            bstats.Add(item);
        }

        public static void ViewBanStats(IUser userobj)
        {
            foreach (BanStatItem item in bstats)
            {
                String str = PrepareBanStat(item);

                if (str != null)
                    Callisto.Print(userobj, str);
            }
        }

        private static String PrepareBanStat(BanStatItem item)
        {
            String str = StringTemplate.GetString(STType.BanStats, item.Index);

            if (str != null)
            {
                str = str.Replace("+n", item.Name);
                str = str.Replace("+ip", item.IP.ToString());
                str = str.Replace("+x", item.Admin == null ? "+x" : item.Admin);
                return str;
            }

            return null;
        }

        public static void InitBanStats()
        {
            bstats = new List<BanStatItem>();
        }

        public static void BanSend(IUser userobj)
        {
            if (last_bansend.Equals(userobj.ExternalIP))
                return;

            last_bansend = userobj.ExternalIP;
            String str = String.Empty;

            switch (RejectionType)
            {
                case Rejections.Age:
                    if (StringTemplate.GetString(STType.BanSend, 0) != null)
                        str = StringTemplate.GetString(STType.BanSend, 0).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.Banned:
                    if (StringTemplate.GetString(STType.BanSend, 1) != null)
                        str = StringTemplate.GetString(STType.BanSend, 1).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.Gender:
                    if (StringTemplate.GetString(STType.BanSend, 2) != null)
                        str = StringTemplate.GetString(STType.BanSend, 2).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.RangeBanned:
                    if (StringTemplate.GetString(STType.BanSend, 3) != null)
                        str = StringTemplate.GetString(STType.BanSend, 3).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.Anon:
                    if (StringTemplate.GetString(STType.BanSend, 4) != null)
                        str = StringTemplate.GetString(STType.BanSend, 4).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.Share:
                    if (StringTemplate.GetString(STType.BanSend, 5) != null)
                        str = StringTemplate.GetString(STType.BanSend, 5).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.ClientFlood:
                    if (StringTemplate.GetString(STType.BanSend, 6) != null)
                        str = StringTemplate.GetString(STType.BanSend, 6).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.LoginFlood:
                    if (StringTemplate.GetString(STType.BanSend, 7) != null)
                        str = StringTemplate.GetString(STType.BanSend, 7).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.NameInUse:
                    if (StringTemplate.GetString(STType.BanSend, 8) != null)
                        str = StringTemplate.GetString(STType.BanSend, 8).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;

                case Rejections.CallistoXFilter:
                    if (StringTemplate.GetString(STType.BanSend, 9) != null)
                        str = StringTemplate.GetString(STType.BanSend, 9).Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString());
                    break;
            }

            if (str.Length > 0)
                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                    if (u.Level > AdminLevel.Regular)
                        if (bs.Contains(u.Id))
                            Callisto.SendPM(u, Callisto.Room.BotName, "REJECT: " + str);
        }

        public static bool AddBanSend(IUser userobj)
        {
            if (bs.Contains(userobj.Id))
                return false;

            bs.Add(userobj.Id);
            return true;
        }

        public static bool RemoveBanSend(IUser userobj)
        {
            if (!bs.Contains(userobj.Id))
                return false;

            bs.Remove(userobj.Id);
            return true;
        }

        public static void AddAntiFlood(IUser userobj)
        {
            if (!af.Contains(userobj.Id))
                af.Add(userobj.Id);
        }

        public static void RemoveAntiFlood(IUser userobj)
        {
            if (af.Contains(userobj.Id))
                af.Remove(userobj.Id);
        }

        public static bool CanFlood(IUser userobj)
        {
            return af.Contains(userobj.Id);
        }

        public static void Update()
        {
            if (!Settings.CbansOn)
                return;

            uint now = Helpers.UnixTimestamp();
            uint target = Settings.CbansLast;
            target += (uint)(Settings.CbansInterval * 3600);

            if (target < now)
            {
                Settings.CbansLast = now;
                MuzzleManager.Clear();
                Callisto.ClearBans();
                KiddyManager.Clear();
                EchoManager.Clear();
                LowerManager.Clear();
                RangeBanManager.Clear();

                if (StringTemplate.GetString(STType.AdminCommands, 2) != null)
                    Callisto.PrintToAdmins(StringTemplate.GetString(STType.AdminCommands, 2), true);

                AdminLog.AddLine("all ban lists have been automatically cleared");
            }
        }

        public static void KillUser(IUser userobj, IUser admin, String args, bool host)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (!host)
                if (StringTemplate.GetString(STType.AdminCommands, 3) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 3).Replace("+n", _admin).Replace("+t", userobj.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

            userobj.Disconnect();
            AdminLog.AddLine(userobj.Name + " was killed by " + admin.Name);
        }

        public static void AddBan(IUser userobj, IUser admin, String args, bool host)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (!host)
            {
                if (StringTemplate.GetString(STType.AdminCommands, 4) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 4).Replace("+n", _admin).Replace("+t", userobj.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

                AdminLog.AddLine(userobj.Name + " was banned by " + admin.Name);
                BanManager.AddBanStatItem(new BanStatItem
                {
                    Name = userobj.Name,
                    IP = userobj.ExternalIP,
                    Admin = admin.Name,
                    Index = 7
                });
            }

            userobj.Ban();
        }

        public static void RemoveBan(IUser remover, int i, bool host)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : remover.Name;
            int counter = 0;

            for (IBan usr = Callisto.FirstBanned; usr != null; usr = usr.NextUser)
            {
                if (counter == i)
                {
                    usr.Banned = false;

                    if (!host)
                        if (StringTemplate.GetString(STType.AdminCommands, 5) != null)
                            Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 5).Replace("+n", _admin).Replace("+t", usr.Name), true);

                    AdminLog.AddLine(usr.Name + " was unbanned by " + remover.Name);
                    break;
                }

                counter++;
            }
        }

        public static void RemoveBan(IUser remover, String args, bool host)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : remover.Name;

            if (!args.StartsWith("\"") || !args.EndsWith("\""))
                return;

            String query = args.Replace("\"", "");

            for (IBan usr = Callisto.FirstBanned; usr != null; usr = usr.NextUser)
            {
                if (usr.Name.StartsWith(query))
                {
                    usr.Banned = false;

                    if (!host)
                        if (StringTemplate.GetString(STType.AdminCommands, 5) != null)
                            Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 5).Replace("+n", _admin).Replace("+t", usr.Name), true);

                    AdminLog.AddLine(usr.Name + " was unbanned by " + remover.Name);
                    break;
                }
            }
        }

        public static void ListBans(IUser asker)
        {
            int counter = 0;
            String str = StringTemplate.GetString(STType.BanList, 1);

            for (IBan usr = Callisto.FirstBanned; usr != null; usr = usr.NextUser)
            {
                if (str != null)
                Callisto.Print(asker, str.Replace("+id", counter.ToString()).Replace("+n", usr.Name).Replace("+ip", usr.ExternalIP.ToString()));
                counter++;
            }

            if (counter == 0)
                if (StringTemplate.GetString(STType.BanList, 0) != null)
                    Callisto.Print(asker, StringTemplate.GetString(STType.BanList, 0));
        }


    }

    class BanStatItem
    {
        public int Index { get; set; }
        public String Name { get; set; }
        public String Admin { get; set; }
        public IPAddress IP { get; set; }
    }
}
