﻿using System;
using System.Net;
using ICallisto;

namespace CallistoX
{
    class Callisto
    {
        private static IHost cback;

        public static RoomSearch RoomSearch { get; set; }

        public static void Set(IHost ih)
        {
            cback = ih;
        }

        public static IPEndPoint[] ServerNodes
        {
            get { return cback.ServerNodes; }
        }

        /// <summary>Find a User Object by exact username</summary>
        public static IUser FindUser(String name)
        {
            for (IUser u = FirstUser; u != null; u = u.NextUser)
                if (u.Name == name)
                    return u;

            if (Linker.IsLinked)
                for (IUser u = FirstLinked; u != null; u = u.NextUser)
                    if (u.Name == name)
                        return u;

            return null;
        }

        /// <summary>Path to the folder available for this plugin to save files</summary>
        public static String DataPath
        {
            get { return cback.DataPath; }
        }

        /// <summary>Send a private message to a user</summary>
        public static void SendPM(IUser target, String sender, object text)
        {
            cback.SendPM(target, sender, text.ToString());
        }

        /// <summary>Send a public message to vroom 0</summary>
        public static void SendText(IUser target, String sender, String text)
        {
            cback.SendText(target, sender, text);
        }

        /// <summary>Send an emote message to vroom 0</summary>
        public static void SendEmote(IUser target, String sender, String text)
        {
            cback.SendEmote(target, sender, text);
        }

        /// <summary>Print a message to everyone</summary>
        public static void Print(object text)
        {
            cback.Print(text.ToString());
        }

        /// <summary>Print a message to everyone [also optionally the linked room]</summary>
        public static void Print(object text, bool link)
        {
            cback.Print(text.ToString(), link);
        }

        /// <summary>Print a message to one user</summary>
        public static void Print(IUser user, object text)
        {
            cback.Print(user, text.ToString());
        }

        /// <summary>Print a message to a vroom</summary>
        public static void Print(ushort vroom, object text)
        {
            cback.Print(vroom, text.ToString());
        }

        /// <summary>Print a message to a vroom [also optionally the linked room]</summary>
        public static void Print(ushort vroom, object text, bool link)
        {
            cback.Print(vroom, text.ToString(), link);
        }

        /// <summary>Print a message to all admins</summary>
        public static void PrintToAdmins(object text)
        {
            cback.PrintToAdmins(text.ToString(), Linker.IsLinked);
        }

        /// <summary>Print a message to all hosts</summary>
        public static void PrintToHosts(object text)
        {
            for (IUser u = FirstUser; u != null; u = u.NextUser)
                if (u.Level == AdminLevel.Host)
                    Print(u, text);
        }

        /// <summary>Print a message to the linked room</summary>
        public static void PrintToLink(object text)
        {
            cback.PrintToLink(text.ToString());
        }

        /// <summary>Print a message to all admins [also optionally the linked admins]</summary>
        public static void PrintToAdmins(object text, bool link)
        {
            cback.PrintToAdmins(text.ToString(), link);
        }

        /// <summary>Set the URL for all users</summary>
        public static void SendURL(String address, object text)
        {
            cback.SendURL(address, text.ToString());
        }

        /// <summary>Get the first user in the local user pool</summary>
        public static IUser FirstUser
        {
            get { return cback.FirstUser; }
        }

        /// <summary>Get the first user in the linked user pool</summary>
        public static IUser FirstLinked
        {
            get { return cback.FirstLinked; }
        }

        /// <summary>Get the first user in the linked user pool</summary>
        public static IBan FirstBanned
        {
            get { return cback.FirstBanned; }
        }

        /// <summary>Clear the server's ban list</summary>
        public static void ClearBans()
        {
            cback.ClearBans();
        }

        /// <summary>Room linking tools</summary>
        public static ILinker Linker
        {
            get { return cback.Linker; }
        }

        /// <summary>Local room statistics</summary>
        public static IStats Stats
        {
            get { return cback.Stats; }
        }

        /// <summary>Local room information and tools</summary>
        public static IRoom Room
        {
            get { return cback.Room; }
        }

        /// <summary>Chatroom hashlink helpers</summary>
        public static IHashlink Hashlink
        {
            get { return cback.Hashlink; }
        }

        /// <summary>Find a user by name</summary>
        public static IUser User(String name)
        {
            return cback.User(name);
        }

        /// <summary>Find a user by id</summary>
        public static IUser User(ushort id)
        {
            return cback.User(id);
        }
    }
}
