﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class CustomNamesManager
    {
        private static List<CustomNamesItem> items = new List<CustomNamesItem>();

        public static void ListCustomNames(IUser userobj)
        {
            foreach (CustomNamesItem i in items)
                Callisto.Print(userobj, "\x000314" + i.name + " : " + i.text);
        }

        public static void AddName(IUser userobj, IUser admin, String text)
        {
            if (userobj.IsLinked)
                return;

            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (admin.Level == AdminLevel.Regular)
                if (userobj.Id != admin.Id)
                    return; // regs can only do their own name

            int i = items.FindIndex(x => x.guid == userobj.Guid && x.name == userobj.Name);

            if (i > -1)
                items[i].text = text;
            else
                items.Add(new CustomNamesItem(userobj.Guid, userobj.Name, text));

            userobj.CustomName = text;
            Save();
            String str = StringTemplate.GetString(STType.AdminCommands, 6);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", userobj.Name), true);

            AdminLog.AddLine(admin.Name + " set custom name for " + userobj.Name);
        }

        public static void RemoveName(IUser userobj, IUser admin)
        {
            if (userobj.IsLinked)
                return;

            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (admin.Level == AdminLevel.Regular)
                if (userobj.Id != admin.Id)
                    return; // regs can only do their own name

            items.RemoveAll(x => x.guid == userobj.Guid && x.name == userobj.Name);
            userobj.CustomName = String.Empty;
            Save();
            String str = StringTemplate.GetString(STType.AdminCommands, 7);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", userobj.Name), true);

            AdminLog.AddLine(admin.Name + " removed custom name for " + userobj.Name);
        }

        public static String SetCustomName(IUser user)
        {
            CustomNamesItem i = items.Find(x => x.guid == user.Guid && x.name == user.Name);

            if (i == null)
                return String.Empty;
            else
                return i.text;
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "customnames.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("customnames");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("guid");
                    String g = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("name");
                    String n = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("text");
                    String t = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    items.Add(new CustomNamesItem(g, n, t));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }

            if (Callisto.Room.IsRunning)
                for (IUser i = Callisto.FirstUser; i != null; i = i.NextUser)
                {
                    String str = SetCustomName(i);

                    if (str.Length > 0)
                        i.CustomName = str;
                }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "customnames.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("customnames");

                foreach (CustomNamesItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.name)));
                    xml.WriteElementString("text", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.text)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class CustomNamesItem
    {
        public String guid;
        public String name;
        public String text;

        public CustomNamesItem(String guid, String name, String text)
        {
            this.guid = guid;
            this.name = name;
            this.text = text;
        }
    }
}
