﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;
using ICallisto;

namespace CallistoX
{
    class DefaultAvatar
    {
        private static byte[] _avatar;

        private static List<String> rem_av_items = new List<String>();

        public static void AddRemAvatar(IUser userobj)
        {
            if (!rem_av_items.Contains(userobj.Guid))
                rem_av_items.Add(userobj.Guid);

            userobj.Avatar = _avatar == null ? null : _avatar;
        }

        public static void AddRemAvatarFilter(IUser userobj)
        {
            if (!rem_av_items.Contains(userobj.Guid))
                rem_av_items.Add(userobj.Guid);
        }

        public static void RemRemAvatar(IUser userobj)
        {
            if (rem_av_items.Contains(userobj.Guid))
                rem_av_items.Remove(userobj.Guid);

            userobj.RestoreAvatar();
        }

        public static void Check()
        {
            if (_avatar != null)
            {
                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                {
                    if (u.Avatar == null)
                    {
                        if (!u.CustomData.Keys.Contains("default_avatar"))
                            u.CustomData["default_avatar"] = (int)0;

                        if (u.CustomData["default_avatar"] is int)
                        {
                            int i = (int)u.CustomData["default_avatar"];

                            if (i++ > 16)
                            {
                                u.Avatar = _avatar;
                                u.CustomData["got_default_avatar"] = true;
                            }

                            u.CustomData["default_avatar"] = i;
                        }
                    }
                }

                // check for disabled avatar users
                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                    if (rem_av_items.Contains(u.Guid))
                        if (!u.Avatar.SequenceEqual<byte>(_avatar))
                            u.Avatar = _avatar;

                if (Callisto.Linker.IsLinked)
                    for (IUser u = Callisto.FirstLinked; u != null; u = u.NextUser)
                        if (rem_av_items.Contains(u.Guid))
                            if (!u.Avatar.SequenceEqual<byte>(_avatar))
                                u.Avatar = _avatar;

            }
            else // check for disabled avatar users
            {
                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                    if (rem_av_items.Contains(u.Guid))
                        if (u.Avatar != null)
                            u.Avatar = null;

                if (Callisto.Linker.IsLinked)
                    for (IUser u = Callisto.FirstLinked; u != null; u = u.NextUser)
                        if (rem_av_items.Contains(u.Guid))
                            if (u.Avatar != null)
                                u.Avatar = null;
            }
        }

        public static bool UpdateDefaultAvatar(String path)
        {
            try
            {
                byte[] raw = File.ReadAllBytes(path);
                Bitmap avatar_raw = new Bitmap(new MemoryStream(raw));

                // create demo pic
                Bitmap avater_demo = new Bitmap(90, 90);
                Graphics g = Graphics.FromImage(avater_demo);
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.DrawImage(avatar_raw, new RectangleF(0, 0, 90, 90));
                MemoryStream ms = new MemoryStream();
                avater_demo.Save(ms, ImageFormat.Jpeg);
                File.WriteAllBytes(Callisto.DataPath + "avatar1.dat", ms.ToArray());
                DemoImage = (Bitmap)avater_demo.Clone();

                // create usable pic
                Bitmap avater_usable = new Bitmap(48, 48);
                g = Graphics.FromImage(avater_usable);
                g.InterpolationMode = InterpolationMode.HighQualityBicubic;
                g.DrawImage(avatar_raw, new RectangleF(0, 0, 48, 48));
                ms = new MemoryStream();
                avater_usable.Save(ms, ImageFormat.Jpeg);
                _avatar = ms.ToArray();
                File.WriteAllBytes(Callisto.DataPath + "avatar2.dat", _avatar);

                // clean up
                ms.Flush();
                ms.Close();

                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                    if (u.CustomData.Keys.Contains("got_default_avatar"))
                        u.Avatar = _avatar;

                return true;
            }
            catch { }

            return false;
        }

        public static bool LoadDefaultAvatar()
        {
            try
            {
                DemoImage = new Bitmap(new MemoryStream(File.ReadAllBytes(Callisto.DataPath + "avatar1.dat")));
                _avatar = File.ReadAllBytes(Callisto.DataPath + "avatar2.dat");
                return true;
            }
            catch
            {
                DemoImage = null;
                _avatar = null;
            }

            return false;
        }

        public static void ClearDefaultAvatar()
        {
            try
            {
                File.Delete(Callisto.DataPath + "avatar1.dat");
                File.Delete(Callisto.DataPath + "avatar2.dat");
            }
            catch { }

            DemoImage = null;
            _avatar = null;

            for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                if (u.CustomData.Keys.Contains("got_default_avatar"))
                    u.RestoreAvatar();
        }

        public static Bitmap ClearedImage
        {
            get
            {
                Bitmap b = new Bitmap(90, 90);
                Graphics g = Graphics.FromImage(b);
                g.FillRectangle(Brushes.White, new Rectangle(0, 0, 90, 90));
                return b;
            }
        }

        public static Bitmap DemoImage { get; private set; }
    }

}
