﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class FileFilters
    {
        private static String[] types = new String[]
        {
            "kill",
            "ban",
            "announce",
            "clone"
        };

        private static List<FilterItem> items = new List<FilterItem>();

        public static void FilterAfter(IUser userobj, String filename)
        {
            String ip = userobj.ExternalIP.ToString();

            foreach (FilterItem i in items)
            {
                if (Regex.IsMatch(filename, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                {
                    switch (i.type)
                    {
                        case "kill":
                            if (StringTemplate.GetString(STType.FilterMessages, 0) != null)
                                Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 0).Replace("+n", userobj.Name).Replace("+x", filename), true);

                            AdminLog.AddLine(userobj.Name + " was killed for sharing " + filename);
                            userobj.Disconnect();
                            return;

                        case "ban":
                            if (StringTemplate.GetString(STType.FilterMessages, 1) != null)
                                Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 1).Replace("+n", userobj.Name).Replace("+x", filename), true);

                            BanManager.AddBanStatItem(new BanStatItem
                            {
                                Name = userobj.Name,
                                IP = userobj.ExternalIP,
                                Admin = null,
                                Index = 0
                            });
                            AdminLog.AddLine(userobj.Name + " was banned for sharing " + filename);
                            userobj.Ban();
                            return;

                        case "announce":
                            Callisto.Print(userobj.Vroom, i.args.Replace("+n", userobj.Name).Replace("+ip", ip).Replace("+f", filename), userobj.Vroom == 0);
                            break;

                        case "clone":
                            if (i.args.StartsWith("/me "))
                                userobj.SendEmote(i.args.Substring(4).Replace("+n", userobj.Name).Replace("+ip", ip).Replace("+f", filename));
                            else
                                userobj.SendText(i.args.Replace("+n", userobj.Name).Replace("+ip", ip).Replace("+f", filename));
                            break;
                    }
                }
            }
        }

        public static void AddFilter(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 2)
                return;

            String trigger = list[0];
            String type = list[1];
            String args = String.Empty;

            list.RemoveRange(0, 2);

            if (list.Count > 0)
                args = String.Join(", ", list.ToArray());

            if (types.Contains(type))
            {
                if (new List<String>(types).IndexOf(type) > 1)
                    if (args.Length == 0)
                        return; // need args

                int exists = items.FindIndex(x => x.trigger == trigger && x.type == type);

                if (exists > -1)
                    items[exists].args = args;
                else
                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });

                String str = StringTemplate.GetString(STType.FilterContent, 0);

                if (str != null)
                    Callisto.Print(str.Replace("+n", _admin).Replace("+t", trigger).Replace("+x", type), true);

                AdminLog.AddLine("file filter added by " + admin.Name + " : " + trigger);
                Save();
            }
        }

        public static void ListFilters(IUser admin)
        {
            if (items.Count == 0)
            {
                if (StringTemplate.GetString(STType.FilterList, 0) != null)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 0));

                return;
            }

            if (StringTemplate.GetString(STType.FilterList, 1) != null)
                for (int i = 0; i < items.Count; i++)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 1).Replace("+id", i.ToString()).Replace("+t", items[i].trigger).Replace("+x", items[i].type));
        }

        public static void RemoveFilter(IUser admin, int ident)
        {
            if (ident < 0 || ident >= items.Count)
                return;

            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            FilterItem item = items[ident];
            items.RemoveAt(ident);

            String str = StringTemplate.GetString(STType.FilterContent, 1);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", item.trigger).Replace("+x", item.type), true);

            AdminLog.AddLine("file filter removed by " + admin.Name + " : " + item.trigger);
            Save();
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "filefilters.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("filefilters");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("trigger");
                    String trigger = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("type");
                    String type = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("args");
                    String args = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "filefilters.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("filefilters");

                foreach (FilterItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("trigger", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.trigger)));
                    xml.WriteElementString("type", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.type)));
                    xml.WriteElementString("args", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.args)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }
}
