﻿using System;
using System.Collections.Generic;
using ICallisto;

namespace CallistoX
{
    class History
    {
        private static List<HistoryItem> items = new List<HistoryItem>();

        public static void AddItem(String name, String text, bool emote)
        {
            if (text.ToUpper().Contains("#LOGIN"))
                return;

            items.Add(new HistoryItem
            {
                name = name,
                text = Helpers.StripColors(text),
                emote = emote,
                time = Helpers.UnixTimestamp()
            });

            if (items.Count > 20)
                items.RemoveAt(0);
        }

        public static void ShowHistory(IUser userobj)
        {
            if (userobj.Level == AdminLevel.Regular) // 10 items
            {
                List<HistoryItem> list = items.Count > 10 ? items.GetRange(items.Count - 11, 10) : items;

                foreach (HistoryItem h in list)
                    if (h.emote)
                        Callisto.SendEmote(userobj, h.name, GetTime(h.time) + h.text);
                    else
                        Callisto.SendText(userobj, h.name, GetTime(h.time) + h.text);
            }
            else // 20 items
            {
                foreach (HistoryItem h in items)
                    if (h.emote)
                        Callisto.SendEmote(userobj, h.name, GetTime(h.time) + h.text);
                    else
                        Callisto.SendText(userobj, h.name, GetTime(h.time) + h.text);
            }

            if (items.Count > 0)
                if (StringTemplate.GetString(STType.ChatHistory, 0) != null)
                    Callisto.Print(userobj, StringTemplate.GetString(STType.ChatHistory, 0));
        }

        private static String GetTime(uint time)
        {
            uint seconds = (Helpers.UnixTimestamp() - time);
            uint minutes = seconds < 60 ? 0 : (seconds / 60);
            seconds = seconds < 60 ? seconds : (seconds - (minutes * 60));
            uint hours = minutes < 60 ? 0 : (minutes / 60);
            minutes = minutes < 60 ? minutes : (minutes - (hours * 60));
            return "[-" + (hours < 10 ? ("0" + hours) : hours.ToString()) + ":" +
                (minutes < 10 ? ("0" + minutes) : minutes.ToString()) + ":" +
                (seconds < 10 ? ("0" + seconds) : seconds.ToString()) + "] ";
        }
    }

    class HistoryItem
    {
        public String name { get; set; }
        public String text { get; set; }
        public bool emote { get; set; }
        public uint time { get; set; }
    }
}
