﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class JoinFilter
    {
        private static String[] types = new String[]
        {
            "disableavatar",
            "censor",
            "vspy",
            "antiflood",
            "ipsend",
            "logsend",
            "bansend",
            "portban",
            "ipban",
            "nameban", // 9
            "move",
            "redirect",
            "pm",
            "announce",
            "clone",
            "changename",
            "reservename",
            "age" // 17
        };

        private static List<FilterItem> items = new List<FilterItem>();

        public static bool FilterBefore(IUser userobj)
        {
            String ip = userobj.ExternalIP.ToString();
            ushort us;

            foreach (FilterItem i in items)
            {
                switch (i.type)
                {
                    case "censor":
                        if (userobj.Level == AdminLevel.Regular)
                            userobj.Name = Regex.Replace(userobj.Name, Regex.Escape(i.trigger), String.Empty, RegexOptions.IgnoreCase);
                        break;

                    case "portban":
                        if (ushort.TryParse(i.trigger, out us))
                            if (userobj.Port == us && userobj.Level == AdminLevel.Regular)
                            {
                                BanManager.AddBanStatItem(new BanStatItem
                                {
                                    Name = userobj.Name,
                                    IP = userobj.ExternalIP,
                                    Admin = null,
                                    Index = 1
                                });
                                AdminLog.AddLine(userobj.Name + " banned by join filter");
                                userobj.Ban();
                                return false;
                            }
                        break;

                    case "ipban":
                        if (ip.StartsWith(i.trigger) && userobj.Level == AdminLevel.Regular)
                        {
                            BanManager.AddBanStatItem(new BanStatItem
                            {
                                Name = userobj.Name,
                                IP = userobj.ExternalIP,
                                Admin = null,
                                Index = 1
                            });
                            AdminLog.AddLine(userobj.Name + " banned by join filter");
                            userobj.Ban();
                            return false;
                        }
                        break;

                    case "nameban":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)) && userobj.Level == AdminLevel.Regular)
                        {
                            BanManager.AddBanStatItem(new BanStatItem
                            {
                                Name = userobj.Name,
                                IP = userobj.ExternalIP,
                                Admin = null,
                                Index = 1
                            });
                            AdminLog.AddLine(userobj.Name + " banned by join filter");
                            userobj.Ban();
                            return false;
                        }
                        break;

                    case "move":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)))
                            if (ushort.TryParse(i.args, out us))
                                userobj.Vroom = us;
                        break;

                    case "redirect":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)) && userobj.Level == AdminLevel.Regular)
                        {
                            userobj.Redirect(i.args);
                            return false;
                        }
                        break;

                    case "changename":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)))
                            userobj.Name = i.args;
                        break;

                    case "reservename":
                        if (userobj.Name == i.trigger)
                            if (ip != i.args)
                                return false;
                        break;

                    case "age":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)))
                        {
                            byte _age;

                            if (byte.TryParse(i.args, out _age))
                                if (_age < 100)
                                    userobj.Age = _age;
                        }
                        break;
                }
            }

            return true;
        }

        public static void FilterAfter(IUser userobj)
        {
            String ip = userobj.ExternalIP.ToString();

            foreach (FilterItem i in items)
            {
                switch (i.type)
                {
                    case "vspy":
                        VSpy.AddUser(userobj);
                        break;

                    case "antiflood":
                        if (ip.StartsWith(i.trigger))
                            BanManager.AddAntiFlood(userobj);
                        break;

                    case "ipsend":
                        if (ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                            IPSendManager.AddIPSend(userobj);
                        break;

                    case "disableavatar":
                        if (ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                            DefaultAvatar.AddRemAvatarFilter(userobj);
                        break;

                    case "logsend":
                        if (ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                            AdminLog.AddToLogSend(userobj);
                        break;

                    case "bansend":
                        if (ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                            BanManager.AddBanSend(userobj);
                        break;

                    case "pm":
                        if (ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                            Callisto.SendPM(userobj, Callisto.Room.BotName, i.args.Replace("+n", userobj.Name).Replace("+ip", ip));
                        break;

                    case "announce":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)))
                            Callisto.Print(userobj.Vroom, i.args.Replace("+n", userobj.Name).Replace("+ip", ip), userobj.Vroom == 0);
                        break;

                    case "clone":
                        if ((ip.StartsWith(i.trigger) || Regex.IsMatch(userobj.Name, Regex.Escape(i.trigger), RegexOptions.IgnoreCase)))
                            if (i.args.StartsWith("/me "))
                                userobj.SendEmote(i.args.Substring(4).Replace("+n", userobj.Name).Replace("+ip", ip));
                            else
                                userobj.SendText(i.args.Replace("+n", userobj.Name).Replace("+ip", ip));
                        break;
                }
            }
        }

        public static void AddFilter(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 2)
                return;

            String trigger = list[0];
            String type = list[1];
            String args = String.Empty;

            list.RemoveRange(0, 2);

            if (list.Count > 0)
                args = String.Join(", ", list.ToArray());

            if (types.Contains(type))
            {
                if (new List<String>(types).IndexOf(type) > 9)
                    if (args.Length == 0)
                        return; // need args

                int exists = items.FindIndex(x => x.trigger == trigger && x.type == type);

                if (exists > -1)
                    items[exists].args = args;
                else
                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });

                String str = StringTemplate.GetString(STType.FilterContent, 2);

                if (str != null)
                    Callisto.Print(str.Replace("+n", _admin).Replace("+t", trigger).Replace("+x", type), true);

                Save();
            }
        }

        public static void ListFilters(IUser admin)
        {
            if (items.Count == 0)
            {
                if (StringTemplate.GetString(STType.FilterList, 0) != null)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 0));

                return;
            }

            if (StringTemplate.GetString(STType.FilterList, 1) != null)
                for (int i = 0; i < items.Count; i++)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 1).Replace("+id", i.ToString()).Replace("+t", items[i].trigger).Replace("+x", items[i].type));
        }

        public static void RemoveFilter(IUser admin, int ident)
        {
            if (ident < 0 || ident >= items.Count)
                return;

            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            FilterItem item = items[ident];
            items.RemoveAt(ident);
            String str = StringTemplate.GetString(STType.FilterContent, 3);

            if (str != null)
                Callisto.Print(str.Replace("+n", _admin).Replace("+t", item.trigger).Replace("+x", item.type), true);

            Save();
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "joinfilters.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("joinfilters");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("trigger");
                    String trigger = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("type");
                    String type = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("args");
                    String args = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "joinfilters.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("joinfilters");

                foreach (FilterItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("trigger", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.trigger)));
                    xml.WriteElementString("type", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.type)));
                    xml.WriteElementString("args", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.args)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }
}
