﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class KiddyManager
    {
        private static List<KiddyItem> items = new List<KiddyItem>();

        public static bool IsKiddied(IUser userobj)
        {
            return items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) != null;
        }

        public static void AddKiddy(IUser admin, IUser target, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.Find(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) == null)
            {
                items.Add(new KiddyItem(target.Guid, target.ExternalIP));
                String str = StringTemplate.GetString(STType.AdminCommands, 10);

                if (str != null)
                    Callisto.Print(str.Replace("+n", _admin).Replace("+t", target.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

                AdminLog.AddLine(target.Name + " kiddied by " + admin.Name);
                Save();
            }
        }

        public static void AddFilterKiddy(IUser target)
        {
            if (items.Find(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) == null)
            {
                items.Add(new KiddyItem(target.Guid, target.ExternalIP));

                if (StringTemplate.GetString(STType.FilterMessages, 2) != null)
                    Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 2).Replace("+n", target.Name), true);

                Save();
            }
        }

        public static void RemoveKiddy(IUser admin, IUser target, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.RemoveAll(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) > 0)
            {
                String str = StringTemplate.GetString(STType.AdminCommands, 11);

                if (str != null)
                    Callisto.Print(str.Replace("+n", _admin).Replace("+t", target.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

                AdminLog.AddLine(target.Name + " unkiddied by " + admin.Name);
                Save();
            }
        }

        public static void Clear()
        {
            items.Clear();
            Save();
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "kiddys.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("kiddys");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("guid");
                    String g = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("ip");
                    IPAddress i = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    items.Add(new KiddyItem(g, i));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "kiddys.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("kiddys");

                foreach (KiddyItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("ip", Convert.ToBase64String(a.ip.GetAddressBytes()));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class KiddyItem
    {
        public String guid;
        public IPAddress ip;

        public KiddyItem(String guid, IPAddress ip)
        {
            this.guid = guid;
            this.ip = ip;
        }
    }
}
