﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class LowerManager
    {
        private static List<LowerItem> items = new List<LowerItem>();
        
        public static void Clear()
        {
            items.Clear();
            Save();
        }

        public static bool HasExceeded(String text) // 75% tolerance
        {
            if (text.Length < 6)
                return false;

            List<char> letters = new List<char>(text.ToCharArray());
            float uppers = (float)letters.FindAll(x => x >= 65 && x <= 90).Count;
            return ((uppers / letters.Count) * 100) >= 75;
        }

        public static bool IsLowered(IUser userobj)
        {
            return items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) != null;
        }

        public static void AddLower(IUser admin, IUser target, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.Find(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) == null)
            {
                items.Add(new LowerItem(target.Guid, target.ExternalIP));

                if (StringTemplate.GetString(STType.AdminCommands, 12) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 12).Replace("+n", _admin).Replace("+t", target.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

                AdminLog.AddLine(target.Name + " lowered by " + admin.Name);
                Save();
            }
        }

        public static void RemoveLower(IUser admin, IUser target, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.RemoveAll(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) > 0)
            {
                if (StringTemplate.GetString(STType.AdminCommands, 13) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 13).Replace("+n", _admin).Replace("+t", target.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

                AdminLog.AddLine(target.Name + " unlowered by " + admin.Name);
                Save();
            }
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "lowers.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("lowers");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("guid");
                    String g = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("ip");
                    IPAddress i = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    items.Add(new LowerItem(g, i));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "lowers.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("lowers");

                foreach (LowerItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("ip", Convert.ToBase64String(a.ip.GetAddressBytes()));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class LowerItem
    {
        public String guid;
        public IPAddress ip;

        public LowerItem(String guid, IPAddress ip)
        {
            this.guid = guid;
            this.ip = ip;
        }
    }
}
