﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class MuzzleManager
    {
        private static List<MuzzleItem> items = new List<MuzzleItem>();

        public static void SetTimeout(IUser userobj, int minutes)
        {
            if (minutes < 0 || minutes > 90)
                return;

            Settings.MuzzleTimeout = minutes;

            if (minutes == 0)
            {
                if (StringTemplate.GetString(STType.MuzzleMessages, 1) != null)
                    Callisto.Print(StringTemplate.GetString(STType.MuzzleMessages, 1).Replace("+n", userobj.Name), true);
            }
            else if (StringTemplate.GetString(STType.MuzzleMessages, 0) != null)
                Callisto.Print(StringTemplate.GetString(STType.MuzzleMessages, 0).Replace("+n", userobj.Name).Replace("+t", minutes.ToString()), true);

            AdminLog.AddLine(userobj.Name + " sets muzzle timeout");
        }

        public static void Update()
        {
            if (Settings.MuzzleTimeout == 0)
                return;

            uint now = Helpers.UnixTimestamp();
            List<MuzzleItem> list = items.FindAll(x => (x.time + (Settings.MuzzleTimeout * 60)) < now);

            if (list.Count > 0)
            {
                if (StringTemplate.GetString(STType.MuzzleMessages, 2) != null)
                    foreach (MuzzleItem m in list)
                        Callisto.Print(StringTemplate.GetString(STType.MuzzleMessages, 2).Replace("+n", m.name), true);

                items.FindAll(x => (x.time + (Settings.MuzzleTimeout * 60)) < now).ForEach(x => TimeoutItem(x));
                items.RemoveAll(x => (x.time + (Settings.MuzzleTimeout * 60)) < now);
                Save();
            }
        }

        public static void TimeoutItem(MuzzleItem m)
        {
            if (Callisto.Room.IsRunning)
                for (IUser i = Callisto.FirstUser; i != null; i = i.NextUser)
                    if (i.Guid == m.guid || i.ExternalIP.Equals(m.ip))
                        i.Muzzled = false;
        }

        public static void AddMuzzle(IUser userobj, IUser admin, String args, bool host)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) != null)
                return;

            items.Add(new MuzzleItem(userobj.Name, userobj.Guid, userobj.ExternalIP, Helpers.UnixTimestamp()));
            userobj.Muzzled = true;

            if (!host)
                if (StringTemplate.GetString(STType.AdminCommands, 14) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 14).Replace("+n", _admin).Replace("+t", userobj.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

            AdminLog.AddLine(userobj.Name + " muzzled by " + admin.Name);
            Save();
        }

        public static void AddFilterMuzzle(IUser userobj)
        {
            if (items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) != null)
                return;

            items.Add(new MuzzleItem(userobj.Name, userobj.Guid, userobj.ExternalIP, Helpers.UnixTimestamp()));
            userobj.Muzzled = true;

            if (StringTemplate.GetString(STType.FilterMessages, 3) != null)
                Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 3).Replace("+n", userobj.Name), true);

            Save();
        }

        public static void RemoveMuzzle(IUser userobj, IUser admin, String args, bool host)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) == null)
                return;

            items.RemoveAll(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP));
            userobj.Muzzled = false;

            if (!host)
                if (StringTemplate.GetString(STType.AdminCommands, 15) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 15).Replace("+n", _admin).Replace("+t", userobj.Name) + (args.Length > 0 ? (" [" + args + "]") : String.Empty), true);

            AdminLog.AddLine(userobj.Name + " unmuzzled by " + admin.Name);
            Save();
        }

        public static bool IsMuzzled(IUser userobj)
        {
            return items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) != null;
        }

        public static void Clear()
        {
            if (Callisto.Room.IsRunning)
                for (IUser i = Callisto.FirstUser; i != null; i = i.NextUser)
                    i.Muzzled = false;

            items.Clear();
            Save();
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "muzzles.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("muzzles");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("name");
                    String n = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("guid");
                    String g = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("ip");
                    IPAddress i = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("time");
                    uint t = uint.Parse(xml.ReadElementContentAsString());
                    items.Add(new MuzzleItem(n, g, i, t));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }

            if (Callisto.Room.IsRunning)
                for (IUser i = Callisto.FirstUser; i != null; i = i.NextUser)
                    i.Muzzled = IsMuzzled(i);
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "muzzles.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("muzzles");

                foreach (MuzzleItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.name)));
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("ip", Convert.ToBase64String(a.ip.GetAddressBytes()));
                    xml.WriteElementString("time", a.time.ToString());
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class MuzzleItem
    {
        public String name;
        public String guid;
        public IPAddress ip;
        public uint time;

        public MuzzleItem(String name, String guid, IPAddress ip, uint time)
        {
            this.name = name;
            this.guid = guid;
            this.ip = ip;
            this.time = time;
        }
    }
}
