﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class PMBlockManager
    {
        private static List<PMBlockItem> items = new List<PMBlockItem>();

        public static void AddUser(IUser userobj)
        {
            if (items.Find(x => x.name == userobj.Name && (x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP))) != null)
                return; // already blocked

            items.Add(new PMBlockItem(userobj.Guid, userobj.ExternalIP, userobj.Name));
            Save();

            if (StringTemplate.GetString(STType.PMBlocking, 0) != null)
                Callisto.Print(StringTemplate.GetString(STType.PMBlocking, 0).Replace("+n", userobj.Name), true);
        }

        public static void RemoveUser(IUser userobj)
        {
            if (items.Find(x => x.name == userobj.Name && (x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP))) == null)
                return; // already unblocked

            items.RemoveAll(x => x.name == userobj.Name && (x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)));
            Save();

            if (StringTemplate.GetString(STType.PMBlocking, 1) != null)
                Callisto.Print(StringTemplate.GetString(STType.PMBlocking, 1).Replace("+n", userobj.Name), true);
        }

        public static bool IsBlocking(IUser target, IUser sender)
        {
            if (items.Find(x => x.name == target.Name && (x.guid == target.Guid || x.ip.Equals(target.ExternalIP))) != null)
            {
                if (StringTemplate.GetString(STType.PMBlocking, 2) != null)
                    Callisto.SendPM(sender, target.Name, StringTemplate.GetString(STType.PMBlocking, 2).Replace("+n", target.Name));
                return true;
            }

            return false;
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "pmblocks.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("pmblocks");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("guid");
                    String g = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("ip");
                    IPAddress i = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("name");
                    String n = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    items.Add(new PMBlockItem(g, i, n));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "pmblocks.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("pmblocks");

                foreach (PMBlockItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("ip", Convert.ToBase64String(a.ip.GetAddressBytes()));
                    xml.WriteElementString("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.name)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class PMBlockItem
    {
        public String guid;
        public IPAddress ip;
        public String name;

        public PMBlockItem(String guid, IPAddress ip, String name)
        {
            this.guid = guid;
            this.ip = ip;
            this.name = name;
        }
    }
}
