﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class PMFilter
    {
        private static String[] types = new String[]
        {
            "kill",
            "ban",
            "censor"
        };

        private static List<FilterItem> items = new List<FilterItem>();

        public static bool FilterBefore(IUser userobj, IUser target, IPMObject pm)
        {
            foreach (FilterItem i in items)
            {
                if (pm.Contains(i.trigger))
                {
                    switch (i.type)
                    {
                        case "kill":
                            if (userobj.IsLinked)
                                return false;

                            if (StringTemplate.GetString(STType.FilterMessages, 4) != null)
                                Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 4).Replace("+n", userobj.Name), true);

                            AdminLog.AddLine(userobj.Name + " killed by pm filter");
                            userobj.Disconnect();
                            return false;

                        case "ban":
                            if (userobj.IsLinked)
                                return false;

                            if (StringTemplate.GetString(STType.FilterMessages, 5) != null)
                                Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 5).Replace("+n", userobj.Name), true);

                            AdminLog.AddLine(userobj.Name + " banned by pm filter");
                            BanManager.AddBanStatItem(new BanStatItem
                            {
                                Name = userobj.Name,
                                IP = userobj.ExternalIP,
                                Admin = null,
                                Index = 3
                            });
                            userobj.Ban();
                            return false;

                        case "censor":
                            if (StringTemplate.GetString(STType.FilterMessages, 6) != null)
                                Callisto.SendPM(userobj, target.Name, StringTemplate.GetString(STType.FilterMessages, 6).Replace("+n", userobj.Name));
                            return false;
                    }
                }
            }

            return true;
        }

        public static void AddFilter(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 2)
                return;

            String trigger = list[0];
            String type = list[1];
            String args = String.Empty;

            list.RemoveRange(0, 2);

            if (list.Count > 0)
                args = String.Join(", ", list.ToArray());

            if (types.Contains(type))
            {
                int exists = items.FindIndex(x => x.trigger == trigger && x.type == type);

                if (exists > -1)
                    items[exists].args = args;
                else
                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });

                if (StringTemplate.GetString(STType.FilterContent, 4) != null)
                    Callisto.Print(StringTemplate.GetString(STType.FilterContent, 4).Replace("+n", _admin).Replace("+t", trigger).Replace("+x", type), true);

                AdminLog.AddLine("pm filter added by " + admin.Name + " : " + trigger);
                Save();
            }
        }

        public static void ListFilters(IUser admin)
        {
            if (items.Count == 0)
            {
                if (StringTemplate.GetString(STType.FilterList, 0) != null)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 0));

                return;
            }

            if (StringTemplate.GetString(STType.FilterList, 1) != null)
                for (int i = 0; i < items.Count; i++)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 1).Replace("+id", i.ToString()).Replace("+t", items[i].trigger).Replace("+x", items[i].type));
        }

        public static void RemoveFilter(IUser admin, int ident)
        {
            if (ident < 0 || ident >= items.Count)
                return;

            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            FilterItem item = items[ident];
            items.RemoveAt(ident);

            if (StringTemplate.GetString(STType.FilterContent, 5) != null)
                Callisto.Print(StringTemplate.GetString(STType.FilterContent, 5).Replace("+n", _admin).Replace("+t", item.trigger).Replace("+x", item.type), true);

            AdminLog.AddLine("pm filter removed by " + admin.Name + " : " + item.trigger);
            Save();
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "pmfilters.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("pmfilters");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("trigger");
                    String trigger = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("type");
                    String type = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("args");
                    String args = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));

                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "pmfilters.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("pmfilters");

                foreach (FilterItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("trigger", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.trigger)));
                    xml.WriteElementString("type", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.type)));
                    xml.WriteElementString("args", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.args)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }
}
