﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class Paint
    {
        private static List<PaintItem> items = new List<PaintItem>();

        public static bool IsPainted(IUser userobj)
        {
            return items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)) != null;
        }

        public static String GetPaint(IUser userobj)
        {
            return items.Find(x => x.guid == userobj.Guid || x.ip.Equals(userobj.ExternalIP)).text;
        }

        public static void AddPaint(IUser admin, IUser target, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.Find(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) == null)
                items.Add(new PaintItem(target.Guid, target.ExternalIP, args));
            else
                items.Find(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)).text = args;

            if (StringTemplate.GetString(STType.AdminCommands, 16) != null)
                Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 16).Replace("+n", _admin).Replace("+t", target.Name), true);

            AdminLog.AddLine(target.Name + " painted by " + admin.Name);
            Save();
        }

        public static void RemovePaint(IUser admin, IUser target)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (items.RemoveAll(x => x.guid == target.Guid || x.ip.Equals(target.ExternalIP)) > 0)
            {
                if (StringTemplate.GetString(STType.AdminCommands, 17) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 17).Replace("+n", _admin).Replace("+t", target.Name), true);

                AdminLog.AddLine(target.Name + " unpainted by " + admin.Name);
                Save();
            }
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "paints.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("paints");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("guid");
                    String g = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("ip");
                    IPAddress i = new IPAddress(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("text");
                    String t = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    items.Add(new PaintItem(g, i, t));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "paints.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("paints");

                foreach (PaintItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("guid", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.guid)));
                    xml.WriteElementString("ip", Convert.ToBase64String(a.ip.GetAddressBytes()));
                    xml.WriteElementString("text", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.text)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class PaintItem
    {
        public String guid;
        public IPAddress ip;
        public String text;

        public PaintItem(String guid, IPAddress ip, String text)
        {
            this.guid = guid;
            this.ip = ip;
            this.text = text;
        }
    }
}
