﻿using System;
using System.Collections.Generic;
using System.Linq;
using ICallisto;

namespace CallistoX
{
    public partial class PluginEntryPoint : IPlugin
    {
        /// <summary>Fires when the plugin is loaded</summary>
        public void OnPluginLoaded(IHost plugin)
        {
            Settings.Load();
            AdminManager.Load();
            MuzzleManager.Load();
            RangeBanManager.Load();
            PMBlockManager.Load();
            CustomNamesManager.Load();
            Topics.Load();
            Vrooms.Load();
            WhowasManager.Load();
            KiddyManager.Load();
            Paint.Load();
            EchoManager.Load();
            LowerManager.Load();
            MotdManager.Load(false);
            ChatLog.Init();
            GreetMsg.Load();
            URLManager.Load();
            WordFilter.Load();
            JoinFilter.Load();
            FileFilters.Load();
            PMFilter.Load();

            this.gui.SetValues();

            if (Callisto.Room.IsRunning)
            {
                Callisto.Print(":: CallistoX Plugin v2.010 Loaded ::");
                this.OnServerStarted();
            }

            if (Settings.TopicClock)
                Topics.ForceClock(true);

            AdminLog.AddLine("CallistoX plugin loaded successfully :-)");
            Callisto.Room.CustomName = Settings.CustomNamesOn;
        }

        /// <summary>Fires when the plugin is terminated</summary>
        public void OnPluginKilled(IHost plugin)
        {
            Settings.Update(true);
            Callisto.RoomSearch.Kill();
        }

        /// <summary>Fires when the server has been started or restarted</summary>
        public void OnServerStarted()
        {
            for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                AdminManager.SetAdminLevel(u);

            BanManager.InitBanStats();
        }

        /// <summary>Fires when the server has been stopped</summary>
        public void OnServerStopped() { }

        /// <summary>Fires every one second</summary>
        public void OnTimer()
        {
            WhowasManager.Update();
            Settings.Update(false);
            BanManager.Update();
            MuzzleManager.Update();
            Topics.UpdateClock();
            URLManager.Update();
            RoomInfo.Check();
            DefaultAvatar.Check();
        }

        /// <summary>Fires before a user has joined</summary>
        public bool OnJoinCheck(IUser userobj)
        {
            AdminManager.SetAdminLevel(userobj);

            if (userobj.Level == AdminLevel.Regular)
            {
                if (RangeBanManager.IsBanned(userobj))
                {
                    BanManager.RejectionType = Rejections.RangeBanned;
                    return false;
                }

                if (Settings.AgeOn && userobj.Age < Settings.Age && userobj.Age > 0)
                {
                    userobj.Ban();
                    BanManager.RejectionType = Rejections.Age;
                    BanManager.AddBanStatItem(new BanStatItem
                    {
                        Name = userobj.Name,
                        IP = userobj.ExternalIP,
                        Admin = null,
                        Index = 2
                    });
                    AdminLog.AddLine(userobj.Name + " banned by age filter");
                    return false;
                }

                bool gender_allowed = true;

                if (userobj.Sex == 0 && !Settings.AllowUnknown)
                    gender_allowed = false;

                if (userobj.Sex == 1 && !Settings.AllowMale)
                    gender_allowed = false;

                if (userobj.Sex == 2 && !Settings.AllowFemale)
                    gender_allowed = false;

                if (!gender_allowed)
                {
                    userobj.Ban();
                    BanManager.RejectionType = Rejections.Gender;
                    BanManager.AddBanStatItem(new BanStatItem
                    {
                        Name = userobj.Name,
                        IP = userobj.ExternalIP,
                        Admin = null,
                        Index = 6
                    });
                    AdminLog.AddLine(userobj.Name + " banned by gender filter");
                    return false;
                }

                if (Settings.AnonOn && userobj.Name.StartsWith("anon "))
                {
                    BanManager.RejectionType = Rejections.Anon;
                    return false;
                }

                if (Settings.ShareOn && !userobj.Browsable)
                {
                    BanManager.RejectionType = Rejections.Share;
                    return false;
                }
            }

            if (Settings.FiltersOn)
            {
                if (!JoinFilter.FilterBefore(userobj))
                {
                    BanManager.RejectionType = Rejections.CallistoXFilter;
                    return false;
                }
            }

            return true;
        }

        /// <summary>Fires when a user was rejected access</summary>
        public void OnRejected(IUser userobj, NativeRejectionType type, ref String msg)
        {
            switch (type)
            {
                case NativeRejectionType.Banned:
                    BanManager.RejectionType = Rejections.Banned;

                    if (StringTemplate.GetString(STType.RejectionMessages, 0) != null)
                        msg = StringTemplate.GetString(STType.RejectionMessages, 0).Replace("+n", userobj.Name);
                    break;

                case NativeRejectionType.ClientFlood:
                    BanManager.RejectionType = Rejections.ClientFlood;

                    if (StringTemplate.GetString(STType.RejectionMessages, 1) != null)
                        msg = StringTemplate.GetString(STType.RejectionMessages, 1).Replace("+n", userobj.Name);
                    break;

                case NativeRejectionType.LoginFlood:
                    BanManager.RejectionType = Rejections.LoginFlood;

                    if (StringTemplate.GetString(STType.RejectionMessages, 2) != null)
                        msg = StringTemplate.GetString(STType.RejectionMessages, 2).Replace("+n", userobj.Name);
                    break;

                case NativeRejectionType.NameInUse:
                    BanManager.RejectionType = Rejections.NameInUse;

                    if (StringTemplate.GetString(STType.RejectionMessages, 3) != null)
                        msg = StringTemplate.GetString(STType.RejectionMessages, 3).Replace("+n", userobj.Name);
                    break;

                case NativeRejectionType.Other:
                    switch (BanManager.RejectionType)
                    {
                        case Rejections.Age:
                            if (StringTemplate.GetString(STType.RejectionMessages, 4) != null)
                                msg = StringTemplate.GetString(STType.RejectionMessages, 4).Replace("+n", userobj.Name);
                            break;

                        case Rejections.Anon:
                            if (StringTemplate.GetString(STType.RejectionMessages, 5) != null)
                                msg = StringTemplate.GetString(STType.RejectionMessages, 5).Replace("+n", userobj.Name);
                            break;

                        case Rejections.Gender:
                            if (StringTemplate.GetString(STType.RejectionMessages, 6) != null)
                                msg = StringTemplate.GetString(STType.RejectionMessages, 6).Replace("+n", userobj.Name);
                            break;

                        case Rejections.RangeBanned:
                            if (StringTemplate.GetString(STType.RejectionMessages, 7) != null)
                                msg = StringTemplate.GetString(STType.RejectionMessages, 7).Replace("+n", userobj.Name);
                            break;

                        case Rejections.Share:
                            if (StringTemplate.GetString(STType.RejectionMessages, 8) != null)
                                msg = StringTemplate.GetString(STType.RejectionMessages, 8).Replace("+n", userobj.Name);
                            break;

                        case Rejections.CallistoXFilter:
                            if (StringTemplate.GetString(STType.RejectionMessages, 9) != null)
                                msg = StringTemplate.GetString(STType.RejectionMessages, 9).Replace("+n", userobj.Name);
                            break;
                    }

                    break;
            }

            if (BanManager.RejectionType != Rejections.Unknown)
                BanManager.BanSend(userobj);

            BanManager.RejectionType = Rejections.Unknown;
        }

        /// <summary>Fires after a user has joined</summary>
        public void OnJoin(IUser userobj)
        {
            if (!userobj.FastPing)
                VSpy.OnJoin(userobj);

            if (!userobj.IsLinked)
            {
                userobj.Muzzled = MuzzleManager.IsMuzzled(userobj);
                userobj.CustomName = CustomNamesManager.SetCustomName(userobj);

                if (!userobj.FastPing)
                {
                    MotdManager.View(userobj);

                    if (Settings.LastSeenOn)
                        WhowasManager.LastSeen(userobj);

                    WhowasManager.AddRecord(userobj);
                }

                if (userobj.Level > AdminLevel.Regular)
                    if (!userobj.FastPing)
                        if (!userobj.CustomClientTags.Contains<String>("CALLISTO_GHOST"))
                            if (StringTemplate.GetString(STType.AdminLogin, 6) != null)
                                Callisto.Print(StringTemplate.GetString(STType.AdminLogin, 6).Replace("+n", userobj.Name).Replace("+x", ((int)userobj.Level).ToString()), true);

                if (Settings.TopicClock)
                    Topics.ForceClock(userobj);

                if (!userobj.FastPing)
                {
                    if (Settings.HistoryOn)
                        History.ShowHistory(userobj);

                    if (Settings.GreetMsgOn)
                        if (!userobj.CustomClientTags.Contains<String>("CALLISTO_GHOST"))
                            GreetMsg.ShowGreetMsg(userobj);

                    if (Settings.PMGreetMsgOn)
                        GreetMsg.ShowPMGreetMsg(userobj);

                    IPSendManager.IPSend(userobj);
                }
            }

            if (!userobj.IsLinked)
                if (Settings.FiltersOn)
                    JoinFilter.FilterAfter(userobj);
        }

        /// <summary>Fires when a user has left the room</summary>
        public void OnPart(IUser userobj)
        {
            VSpy.OnPart(userobj);

            if (!userobj.IsLinked)
            {
                BanManager.RemoveBanSend(userobj);
                AdminLog.RemoveFromLogSend(userobj, false);
                VSpy.RemoveUser(userobj, true);
                BanManager.RemoveAntiFlood(userobj);
                IPSendManager.RemoveIPSend(userobj);
            }
        }

        /// <summary>Fires before OnTextBefore (ignores custom names etc)</summary>
        public void OnTextReceived(IUser userobj, String text)
        {
            if (userobj.Muzzled)
            {
                if (StringTemplate.GetString(STType.MuzzleMessages, 3) != null)
                    Callisto.PrintToHosts(StringTemplate.GetString(STType.MuzzleMessages, 3).Replace("+n", userobj.Name).Replace("+t", Helpers.StripColors(text)));

                if (!userobj.IsLinked)
                    if (StringTemplate.GetString(STType.MuzzleMessages, 5) != null)
                        Callisto.Print(userobj, StringTemplate.GetString(STType.MuzzleMessages, 5).Replace("+n", userobj.Name));

                return;
            }

            if (Settings.ChatLogOn)
                ChatLog.AddLine(userobj.Name + "> " + text);

            if (userobj.IsLinked)
                History.AddItem(userobj.Name, text, false);

            VSpy.OnText(userobj, text);
        }

        /// <summary>Fires before a user's text message is published</summary>
        public String OnTextBefore(IUser userobj, String text) 
        {
            List<String> hiders = new List<String>(new String[]
            {
                "#whisper", "#announce", "#clone", "#host"
            }); // always invisible

            if (hiders.Find(x => text.StartsWith(x)) != null)
                return String.Empty;

            if (userobj.Level > AdminLevel.Regular)
                if (Settings.StealthOn)
                    if (text.StartsWith("#"))
                        return String.Empty;

            if ((!text.StartsWith("#") && userobj.Level > AdminLevel.Regular) || userobj.Level == AdminLevel.Regular)
                if (TranslatorManager.Translate(userobj, text))
                    return String.Empty;

            hiders = new List<String>(new String[]
            {
                "#addwordfilter", "#addline", "#editline", "#remline", "#viewfilter"
            }); // wordfilter builders

            if (Settings.FiltersOn)
                if (!(hiders.Find(x => text.StartsWith(x)) != null && userobj.Level > AdminLevel.Regular))
                    text = WordFilter.FilterBefore(userobj, text, false);

            if (String.IsNullOrEmpty(text))
                return String.Empty;

            if (!userobj.IsLinked)
                History.AddItem(userobj.Name, text, false);

            IdleManager.Remove(userobj);

            if (Paint.IsPainted(userobj))
                text = Paint.GetPaint(userobj) + text;

            if (!Settings.ColorsOn && userobj.Level == AdminLevel.Regular)
                text = Helpers.StripColors(text);

            if (LowerManager.IsLowered(userobj))
                text = text.ToLower();

            if (Settings.CapsOn && userobj.Level == AdminLevel.Regular)
                if (LowerManager.HasExceeded(text))
                    text = text.ToLower();

            if (KiddyManager.IsKiddied(userobj))
                return "\x000313" + Helpers.StripColors(text) + "(A)";

            return text;
        }

        /// <summary>Fires after a user's text message is published</summary>
        public void OnTextAfter(IUser userobj, String text)
        {
            if (EchoManager.IsEchoed(userobj))
                userobj.SendText(EchoManager.GetEcho(userobj));
        }

        /// <summary>Fires before OnEmoteBefore (ignores custom names etc)</summary>
        public void OnEmoteReceived(IUser userobj, String text)
        {
            if (userobj.Muzzled)
            {
                if (StringTemplate.GetString(STType.MuzzleMessages, 4) != null)
                    Callisto.PrintToHosts(StringTemplate.GetString(STType.MuzzleMessages, 4).Replace("+n", userobj.Name).Replace("+t", Helpers.StripColors(text)));

                if (!userobj.IsLinked)
                    if (StringTemplate.GetString(STType.MuzzleMessages, 5) != null)
                        Callisto.Print(userobj, StringTemplate.GetString(STType.MuzzleMessages, 5).Replace("+n", userobj.Name));

                return;
            }

            if (Settings.ChatLogOn)
                ChatLog.AddLine("* " + userobj.Name + " " + text);

            if (userobj.IsLinked)
                History.AddItem(userobj.Name, text, true);

            VSpy.OnText(userobj, text);
        }

        /// <summary>Fires before a user's emote message is published</summary>
        public String OnEmoteBefore(IUser userobj, String text)
        {
            if (TranslatorManager.IsTranslated(userobj))
                return String.Empty;

            if (Settings.FiltersOn)
                text = WordFilter.FilterBefore(userobj, text, true);

            if (String.IsNullOrEmpty(text))
                return String.Empty;

            if (!userobj.IsLinked)
                History.AddItem(userobj.Name, text, true);

            IdleManager.Remove(userobj);

            if (text.StartsWith("idles"))
                IdleManager.Add(userobj);

            if (LowerManager.IsLowered(userobj))
                text = text.ToLower();

            if (Settings.CapsOn && userobj.Level == AdminLevel.Regular)
                if (LowerManager.HasExceeded(text))
                    text = text.ToLower();

            return text;
        }

        /// <summary>Fires after a user's emote message is published</summary>
        public void OnEmoteAfter(IUser userobj, String text)
        {
            if (EchoManager.IsEchoed(userobj))
                userobj.SendEmote(EchoManager.GetEcho(userobj));
        }

        public void OnCaptcha(IUser userobj, CaptchaState state)
        {
            switch (state)
            {
                case CaptchaState.Exceeded:
                    if (StringTemplate.GetString(STType.Captcha, 0) != null)
                        Callisto.Print(userobj, StringTemplate.GetString(STType.Captcha, 0));
                    break;

                case CaptchaState.Incorrect:
                    if (StringTemplate.GetString(STType.Captcha, 1) != null)
                        Callisto.Print(userobj, StringTemplate.GetString(STType.Captcha, 1));
                    break;

                case CaptchaState.Sending:
                    if (StringTemplate.GetString(STType.Captcha, 2) != null)
                        Callisto.Print(userobj, StringTemplate.GetString(STType.Captcha, 2));
                    break;

                case CaptchaState.Success:
                    if (StringTemplate.GetString(STType.Captcha, 3) != null)
                        Callisto.Print(userobj, StringTemplate.GetString(STType.Captcha, 3));
                    break;
            }
        }

        /// <summary>Fires when a user submits a room command</summary>
        public void OnCommand(IUser userobj, String command_text, IUser target, String args)
        {
            if (!Settings.EnableAdmins && userobj.Level != AdminLevel.Host)
            {
                if (StringTemplate.GetString(STType.AdminMessages, 14) != null)
                    Callisto.Print(userobj, StringTemplate.GetString(STType.AdminMessages, 14).Replace("+n", userobj.Name));
                return;
            }

            int i;
            ushort u;

            if (command_text.StartsWith("pmblock "))
            {
                if (command_text.Substring(8) == "on")
                    PMBlockManager.AddUser(userobj);
                else if (command_text.Substring(8) == "off")
                    PMBlockManager.RemoveUser(userobj);
            }

            if (command_text == "info")
                CmdProcessor.Info(userobj, false);

            if (command_text == "pminfo")
                CmdProcessor.Info(userobj, true);

            if (command_text == "linkhash")
                Linking.LinkHash(userobj);

            if (command_text == "id")
                Callisto.Print(userobj, "\x000314" + userobj.Name + ": " + userobj.Id);

            if (command_text == "idle" || command_text == "idles")
                IdleManager.Add(userobj);

            if (Settings.GeneralOn || userobj.Level > AdminLevel.Regular) // general
            {
                if (command_text.StartsWith("customname "))
                    if (target != null)
                        CustomNamesManager.AddName(target, userobj, args);

                if (command_text.StartsWith("uncustomname "))
                    if (target != null)
                        CustomNamesManager.RemoveName(target, userobj);

                if (command_text.StartsWith("vroom "))
                    Vrooms.ProcessVroom(userobj, command_text.Substring(6));

                if (command_text.StartsWith("google "))
                    CmdProcessor.Google(userobj, command_text.Substring(7));

                if (command_text.StartsWith("whisper "))
                    if (target != null)
                        CmdProcessor.Whisper(userobj, target, args);

                if (command_text == "locate")
                    Vrooms.Locate(userobj);

                if (command_text == "viewmotd")
                    MotdManager.View(userobj);
            } // end of general

            if (userobj.Level > AdminLevel.Regular)
            {
                if (target != null)
                {
                    if (target.Level < userobj.Level)
                    {
                        if (command_text.StartsWith("muzzle ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.muzzle))
                            MuzzleManager.AddMuzzle(target, userobj, args, false);

                        if (command_text.StartsWith("unmuzzle ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unmuzzle))
                            MuzzleManager.RemoveMuzzle(target, userobj, args, false);

                        if ((command_text.StartsWith("kill ") || command_text.StartsWith("kick ")) && userobj.Level >= CmdLevelManager.GetLevel(Cmd.kill))
                            BanManager.KillUser(target, userobj, args, false);

                        if (command_text.StartsWith("ban ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.ban))
                            BanManager.AddBan(target, userobj, args, false);
                    } // end of commands on lower admins and users

                    if (command_text.StartsWith("clone ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.clone))
                        CmdProcessor.Clone(userobj, target, args, false);

                    if (command_text.StartsWith("changename ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.changename))
                        CmdProcessor.ChangeName(userobj, target, args);

                    if (command_text.StartsWith("oldname ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.oldname))
                        CmdProcessor.OldName(userobj, target);

                    if (command_text.StartsWith("move ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.move))
                        if (ushort.TryParse(args, out u))
                            CmdProcessor.Move(userobj, target, u);

                    if (command_text.StartsWith("whois ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.whois))
                        CmdProcessor.Whois(userobj, target);

                    if (command_text.StartsWith("kiddy ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.kiddy))
                        KiddyManager.AddKiddy(userobj, target, args);

                    if (command_text.StartsWith("unkiddy ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unkiddy))
                        KiddyManager.RemoveKiddy(userobj, target, args);

                    if (command_text.StartsWith("paint ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.paint))
                        Paint.AddPaint(userobj, target, args);

                    if (command_text.StartsWith("unpaint ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unpaint))
                        Paint.RemovePaint(userobj, target);

                    if (command_text.StartsWith("echo ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.echo))
                        EchoManager.AddEcho(userobj, target, args);

                    if (command_text.StartsWith("unecho ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unecho))
                        EchoManager.RemoveEcho(userobj, target);

                    if (command_text.StartsWith("lower ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.lower))
                        LowerManager.AddLower(userobj, target, args);

                    if (command_text.StartsWith("unlower ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unlower))
                        LowerManager.RemoveLower(userobj, target, args);

                    if (command_text.StartsWith("redirect ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.redirect))
                        CmdProcessor.Redirect(userobj, target, args);

                    if (command_text.StartsWith("trace ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.trace))
                    {
                        CmdProcessor.Trace(userobj, target);
                        return;
                    }

                    if (command_text.StartsWith("weather ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.weather))
                        CmdProcessor.GetWeather(userobj, target);

                    if (command_text.StartsWith("translate ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.translate))
                        TranslatorManager.AddItem(userobj, target, args);

                    if (command_text.StartsWith("untranslate ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.untranslate))
                        TranslatorManager.RemoveItem(userobj, target);

                    if (command_text.StartsWith("enableavatar ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.enableavatar))
                        CmdProcessor.EnableAvatar(userobj, target);

                    if (command_text.StartsWith("disableavatar ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.disableavatar))
                        CmdProcessor.DisableAvatar(userobj, target);
                } // end of commands on a target

                if (command_text == "clearscreen" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.clearscreen))
                    CmdProcessor.ClearScreen(userobj);

                if (command_text == "banstats" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.banstats))
                    BanManager.ViewBanStats(userobj);

                if (command_text.StartsWith("trace ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.trace))
                    CmdProcessor.Trace(userobj, command_text.Substring(6).Replace("\"", ""));

                if (command_text == "listbans" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.listbans))
                    BanManager.ListBans(userobj);

                if (command_text.StartsWith("unban ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unban))
                {
                    if (int.TryParse(command_text.Substring(6), out i))
                        BanManager.RemoveBan(userobj, i, false);
                    else BanManager.RemoveBan(userobj, command_text.Substring(6), false);
                }

                if (command_text == "listrangebans" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.listrangebans))
                    RangeBanManager.ListBans(userobj);

                if (command_text.StartsWith("rangeban ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.rangeban))
                    RangeBanManager.AddBan(userobj, command_text.Substring(9));

                if (command_text.StartsWith("rangeunban ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.rangeunban))
                {
                    if (int.TryParse(command_text.Substring(11), out i))
                        RangeBanManager.RemoveBan(userobj, i);
                    else RangeBanManager.RemoveBan(userobj, command_text.Substring(11));
                }

                if (command_text.StartsWith("urban ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.urban))
                    CmdProcessor.Urban(userobj, command_text.Substring(6));

                if (command_text.StartsWith("define ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.define))
                    CmdProcessor.Define(userobj, command_text.Substring(7));

                if (command_text == "customnames" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.customnames))
                    CustomNamesManager.ListCustomNames(userobj);

                if (command_text.StartsWith("cloak ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.cloak))
                    CmdProcessor.CloakOn(userobj, command_text.Substring(6));

                if (command_text.StartsWith("customnames ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.customnames))
                    CmdProcessor.CustomNamesOn(userobj, command_text.Substring(12));

                if (command_text.StartsWith("general ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.general))
                    CmdProcessor.GeneralOn(userobj, command_text.Substring(8));

                if (command_text.StartsWith("caps ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.caps))
                    CmdProcessor.CapsOn(userobj, command_text.Substring(5));

                if (command_text.StartsWith("colors ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.colors))
                    CmdProcessor.ColorsOn(userobj, command_text.Substring(7));

                if (command_text.StartsWith("lastseen ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.lastseen))
                    CmdProcessor.LastSeenOn(userobj, command_text.Substring(9));

                if (command_text.StartsWith("idle ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.idle))
                    CmdProcessor.IdleOn(userobj, command_text.Substring(5));

                if (command_text.StartsWith("shout ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.shout))
                    CmdProcessor.Shout(userobj, command_text.Substring(6));

                if (command_text.StartsWith("announce ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.announce))
                    CmdProcessor.Announce(userobj, command_text.Substring(9));

                if (command_text == "admins" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.admins))
                    CmdProcessor.ListAdmins(userobj);

                if (command_text.StartsWith("adminmsg ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.adminmsg))
                    if (StringTemplate.GetString(STType.AdminMessages, 20) != null)
                        Callisto.PrintToAdmins(StringTemplate.GetString(STType.AdminMessages, 20).Replace("+n", userobj.Name).Replace("+x", command_text.Substring(9)), true);

                if (command_text == "stats" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.stats))
                    CmdProcessor.Stats(userobj);

                if (command_text.StartsWith("whowas ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.whowas))
                    WhowasManager.Whowas(userobj, command_text.Substring(7));

                if (command_text.StartsWith("roomsearch ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.roomsearch))
                    CmdProcessor.RoomSearch(userobj, command_text.Substring(11));

                if (command_text.StartsWith("mtimeout ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.mtimeout))
                    if (int.TryParse(command_text.Substring(9), out i))
                        MuzzleManager.SetTimeout(userobj, i);

                if (command_text.StartsWith("clock ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.clock))
                    CmdProcessor.TopicClockOn(userobj, command_text.Substring(6));

                if (command_text.StartsWith("adminannounce ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.adminannounce))
                    CmdProcessor.AdminAnnounceOn(userobj, command_text.Substring(14));

                if (command_text.StartsWith("filter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.filter))
                    CmdProcessor.FiltersOn(userobj, command_text.Substring(7));

                if (command_text.StartsWith("addjoinfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addjoinfilter))
                    JoinFilter.AddFilter(userobj, command_text.Substring(14));

                if (command_text.StartsWith("remjoinfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.remjoinfilter))
                    if (int.TryParse(command_text.Substring(14), out i))
                        JoinFilter.RemoveFilter(userobj, i);

                if (command_text == "joinfilters" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.joinfilters))
                    JoinFilter.ListFilters(userobj);

                if (command_text.StartsWith("addfilefilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addfilefilter))
                    FileFilters.AddFilter(userobj, command_text.Substring(14));

                if (command_text.StartsWith("remfilefilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.remfilefilter))
                    if (int.TryParse(command_text.Substring(14), out i))
                        FileFilters.RemoveFilter(userobj, i);

                if (command_text == "filefilters" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.filefilters))
                    FileFilters.ListFilters(userobj);

                if (command_text.StartsWith("addpmfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addpmfilter))
                    PMFilter.AddFilter(userobj, command_text.Substring(12));

                if (command_text.StartsWith("rempmfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.rempmfilter))
                    if (int.TryParse(command_text.Substring(12), out i))
                        PMFilter.RemoveFilter(userobj, i);

                if (command_text == "pmfilters" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.pmfilters))
                    PMFilter.ListFilters(userobj);

                if (command_text.StartsWith("addwordfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addwordfilter))
                    WordFilter.AddFilter(userobj, command_text.Substring(14));

                if (command_text.StartsWith("remwordfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.remwordfilter))
                    if (int.TryParse(command_text.Substring(14), out i))
                        WordFilter.RemoveFilter(userobj, i);

                if (command_text.StartsWith("viewfilter ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.viewfilter))
                    WordFilter.ViewFilter(userobj, command_text.Substring(11));

                if (command_text.StartsWith("addline ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addline))
                    WordFilter.AddLine(userobj, command_text.Substring(8));

                if (command_text.StartsWith("editline ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.editline))
                    WordFilter.EditLine(userobj, command_text.Substring(9));

                if (command_text.StartsWith("remline ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.remline))
                    WordFilter.RemLine(userobj, command_text.Substring(8));

                if (command_text == "wordfilters" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.wordfilters))
                    WordFilter.ListFilters(userobj);

                if (command_text.StartsWith("bansend ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.bansend))
                    CmdProcessor.BanSendOn(userobj, command_text.Substring(8));

                if (command_text.StartsWith("logsend ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.logsend))
                    CmdProcessor.LogSendOn(userobj, command_text.Substring(8));

                if (command_text.StartsWith("stealth ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.stealth))
                    CmdProcessor.StealthOn(userobj, command_text.Substring(8));

                if (command_text.StartsWith("ipsend ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.ipsend))
                    CmdProcessor.IPSendOn(userobj, command_text.Substring(7));

                if (command_text.StartsWith("sharefiles ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.sharefiles))
                    CmdProcessor.ShareFileOn(userobj, command_text.Substring(11));

                if (command_text.StartsWith("anon ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.anon))
                    CmdProcessor.AnonOn(userobj, command_text.Substring(5));

                if (command_text.StartsWith("topic ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.topic))
                    Topics.SetTopic(userobj, command_text.Substring(6));

                if (command_text.StartsWith("greetmsg ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.greetmsg))
                    GreetMsg.GreetMsgOn(userobj, command_text.Substring(9));

                if (command_text.StartsWith("pmgreetmsg ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.pmgreetmsg))
                    GreetMsg.PMGreetMsgOn(userobj, command_text.Substring(11));

                if (command_text.StartsWith("setpmgreetmsg ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.setpmgreetmsg))
                    GreetMsg.SetPMGreetMsg(userobj, command_text.Substring(14));

                if (command_text.StartsWith("addgreetmsg ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addgreetmsg))
                    GreetMsg.AddGreetMsg(userobj, command_text.Substring(12));

                if (command_text.StartsWith("remgreetmsg ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.remgreetmsg))
                    if (int.TryParse(command_text.Substring(12), out i))
                        GreetMsg.RemGreetMsg(userobj, i);

                if (command_text.StartsWith("listgreetmsg") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.listgreetmsg))
                    GreetMsg.ListGreetMsg(userobj);

                if (command_text.StartsWith("url ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.url))
                    URLManager.URLOn(userobj, command_text.Substring(4));

                if (command_text.StartsWith("addurl ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.addurl))
                    URLManager.AddItem(userobj, command_text.Substring(7));

                if (command_text.StartsWith("remurl ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.remurl))
                    if (int.TryParse(command_text.Substring(7), out i))
                        URLManager.RemoveItem(userobj, i);

                if (command_text.StartsWith("listurl") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.listurl))
                    URLManager.ListURLs(userobj);

                if (command_text.StartsWith("vspy ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.vspy))
                    CmdProcessor.VspyOn(userobj, command_text.Substring(5));

                if (command_text == "listlang" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.listlang))
                    TranslatorManager.ListLang(userobj);

                if (command_text == "filterhelp")
                    HelpMessages.FilterHelp(userobj);

                if (command_text.StartsWith("link ") && userobj.Level >= CmdLevelManager.GetLevel(Cmd.link))
                    Linking.LinkConnect(userobj, command_text.Substring(5));

                if (command_text == "unlink" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.unlink))
                    Linking.LinkDisconnect(userobj);

                if (command_text == "linkinfo" && userobj.Level >= CmdLevelManager.GetLevel(Cmd.linkinfo))
                    Linking.LinkInfo(userobj);
            } // end of admin commands

            if (userobj.Level == AdminLevel.Host) // host only commands
            {
                if (command_text.StartsWith("roominfo "))
                    RoomInfo.RoomInfoOn(userobj, command_text.Substring(9));

                if (command_text.StartsWith("status "))
                    RoomInfo.Status(userobj, command_text.Substring(7));

                if (command_text == "enableadmins")
                    AdminManager.EnableAdmin(userobj, true);

                if (command_text == "disableadmins")
                    AdminManager.EnableAdmin(userobj, false);

                if (command_text.StartsWith("history "))
                    CmdProcessor.HistoryOn(userobj, command_text.Substring(8));

                if (command_text.StartsWith("cban"))
                    CmdProcessor.ClearBans(userobj, false);

                if (command_text.StartsWith("addautologin "))
                    if (int.TryParse(args, out i) && target != null)
                        AdminManager.AddAdmin(userobj, target, i);

                if (command_text.StartsWith("remautologin "))
                    if (int.TryParse(command_text.Substring(13), out i))
                        AdminManager.RemoveAdmin(userobj, i);

                if (command_text == "autologins")
                    AdminManager.ListAdmin(userobj);

                if (command_text.StartsWith("grant "))
                    if (int.TryParse(args, out i))
                        AdminManager.TempGrant(userobj, target, i);

                if (command_text.StartsWith("ungrant "))
                    AdminManager.TempUngrant(userobj, target);

                if (command_text == "loadmotd")
                    MotdManager.Load(true);

                if (command_text.StartsWith("hostunban "))
                {
                    if (int.TryParse(command_text.Substring(10), out i))
                        BanManager.RemoveBan(userobj, i, true);
                    else BanManager.RemoveBan(userobj, command_text.Substring(10), true);
                }

                if (command_text.StartsWith("hostcban"))
                    CmdProcessor.ClearBans(userobj, true);

                if (target != null)
                {
                    if (command_text.StartsWith("hostclone "))
                        CmdProcessor.Clone(userobj, target, args, true);

                    if (command_text.StartsWith("hostkill ") || command_text.StartsWith("hostkick "))
                        BanManager.KillUser(target, userobj, args, true);

                    if (command_text.StartsWith("hostban "))
                        BanManager.AddBan(target, userobj, args, true);

                    if (command_text.StartsWith("hostmuzzle "))
                        MuzzleManager.AddMuzzle(target, userobj, args, true);

                    if (command_text.StartsWith("hostunmuzzle "))
                        MuzzleManager.RemoveMuzzle(target, userobj, args, true);
                }
            } // end of host commands
        }

        /// <summary>Fires when a user types #help</summary>
        public void OnHelp(IUser userobj)
        {
            HelpMessages.PrintHelp(userobj);
        }

        /// <summary>Fires before a user has changed vroom</summary>
        public bool OnVroomJoinCheck(IUser userobj, ushort vroom)
        {
            return Vrooms.CanJoin(userobj, vroom);
        }

        /// <summary>Fires after a user has changed vroom</summary>
        public void OnVroomJoin(IUser userobj)
        {
            Topics.GetTopic(userobj);
        }

        /// <summary>Fires after a user has left a vroom</summary>
        public void OnVroomPart(IUser userobj) { }

        /// <summary>Fires before a user is about to flood out of the room</summary>
        public bool OnFlood(IUser userobj, ProtoMessage msg)
        {
            if (BanManager.CanFlood(userobj))
                return false;

            if (StringTemplate.GetString(STType.AdminMessages, 15) != null)
                Callisto.Print(userobj, StringTemplate.GetString(STType.AdminMessages, 15).Replace("+n", userobj.Name));

            if (StringTemplate.GetString(STType.AdminMessages, 16) != null)
                Callisto.PrintToAdmins(StringTemplate.GetString(STType.AdminMessages, 16).Replace("+n", userobj.Name).Replace("+x", msg.ToString()));

            return true;
        }

        /// <summary>Fires after a user has submitted a shared file</summary>
        public void OnFileReceived(IUser userobj, String filename, MimeMessage msg)
        {
            MimeMessage[] filtered_types = new MimeMessage[]
            {
                MimeMessage.ARES_MIME_IMAGE,
                MimeMessage.ARES_MIME_OTHER,
                MimeMessage.ARES_MIME_SOFTWARE,
                MimeMessage.ARES_MIME_VIDEO
            };

            if (userobj.Level == AdminLevel.Regular && Settings.FiltersOn)
                if (Array.IndexOf(filtered_types, msg) > -1)
                    FileFilters.FilterAfter(userobj, filename);
        }

        /// <summary>Fires before a user publishes a private message</summary>
        public bool OnPM(IUser userobj, IUser target, IPMObject pm)
        {
            if (userobj.Level == AdminLevel.Regular || (userobj.Level > AdminLevel.Regular && userobj.Level < target.Level))
                if (PMBlockManager.IsBlocking(target, userobj))
                    return false;

            if (userobj.Level == AdminLevel.Regular && Settings.FiltersOn)
                return PMFilter.FilterBefore(userobj, target, pm);

            return true;
        }

        /// <summary>Fires after a user has logged in or attempted to log in</summary>
        public void OnLoginRequested(IUser userobj)
        {
            if (userobj.Level > AdminLevel.Regular)
            {
                if (StringTemplate.GetString(STType.AdminLogin, 6) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminLogin, 6).Replace("+n", userobj.Name).Replace("+x", ((int)userobj.Level).ToString()), true);

                AdminManager.ClearInvalidAttempts(userobj);
            }
            else AdminManager.InvalidLoginAttempt(userobj);
        }

        /// <summary>Fires when this plugin throws an exception</summary>
        public void OnError(String plugin_name, Exception e) { }

        /// <summary>Fires when a room link has been established</summary>
        public void OnLinkJoin(ILink room)
        {
            if (StringTemplate.GetString(STType.Linking, 6) != null)
                Callisto.Print(StringTemplate.GetLinkString(STType.Linking, 6).Replace("+r", room.Name));
        }

        /// <summary>Fires when a room link has been disconnected</summary>
        public void OnLinkPart(ILink room)
        {
            if (StringTemplate.GetString(STType.Linking, 7) != null)
                Callisto.Print(StringTemplate.GetLinkString(STType.Linking, 7).Replace("+r", room.Name));
        }

        /// <summary>Fires when a room link throws an exception</summary>
        public void OnLinkError(ILink room, LinkError code)
        {
            switch (code)
            {
                case LinkError.LinkAdminDisabled:
                    if (StringTemplate.GetString(STType.Linking, 8) != null)
                        Callisto.PrintToAdmins(StringTemplate.GetLinkString(STType.Linking, 8).Replace("+r", room.Name));
                    break;

                default:
                    if (StringTemplate.GetString(STType.Linking, 9) != null)
                        Callisto.Print(StringTemplate.GetLinkString(STType.Linking, 9).Replace("+r", room.Name).Replace("+n", code.ToString()));
                    break;
            }
        }

        /// <summary>Fires when a linked admin submites a room command</summary>
        public void OnLinkCommand(IUser userobj, String command, IUser target, String args)
        {
            ushort u;

            if (!Settings.EnableAdmins)
            {
                if (StringTemplate.GetString(STType.AdminMessages, 14) != null)
                    Callisto.Print(userobj, StringTemplate.GetString(STType.AdminMessages, 14).Replace("+n", userobj.Name));
                return;
            }

            if (target != null)
            {
                if (target.Level == AdminLevel.Regular)
                {
                    if (command == "muzzle")
                        MuzzleManager.AddMuzzle(target, userobj, args, false);

                    if (command == "kill" || command == "kick")
                        BanManager.KillUser(target, userobj, args, false);

                    if (command == "ban")
                        BanManager.AddBan(target, userobj, args, false);

                    if (command == "move")
                        if (ushort.TryParse(args, out u))
                            CmdProcessor.Move(userobj, target, u);
                } // end of commands on non admins

                if (command == "unmuzzle")
                    MuzzleManager.RemoveMuzzle(target, userobj, args, false);

                if (command == "clone")
                    CmdProcessor.Clone(userobj, target, args, false);

                if (command == "changename")
                    CmdProcessor.ChangeName(userobj, target, args);

                if (command == "oldname")
                    CmdProcessor.OldName(userobj, target);

                if (command == "whois")
                    CmdProcessor.Whois(userobj, target);

                if (command == "kiddy")
                    KiddyManager.AddKiddy(userobj, target, args);

                if (command == "unkiddy")
                    KiddyManager.RemoveKiddy(userobj, target, args);

                if (command == "paint")
                    Paint.AddPaint(userobj, target, args);

                if (command == "unpaint")
                    Paint.RemovePaint(userobj, target);

                if (command == "echo")
                    EchoManager.AddEcho(userobj, target, args);

                if (command == "unecho")
                    EchoManager.RemoveEcho(userobj, target);

                if (command == "lower")
                    LowerManager.AddLower(userobj, target, args);

                if (command == "unlower")
                    LowerManager.RemoveLower(userobj, target, args);

                if (command == "redirect")
                    CmdProcessor.Redirect(userobj, target, args);

                if (command == "trace")
                    CmdProcessor.Trace(userobj, target);

                if (command == "weather")
                    CmdProcessor.Trace(userobj, target);

                if (command == "translate")
                    TranslatorManager.AddItem(userobj, target, args);

                if (command == "untranslate")
                    TranslatorManager.RemoveItem(userobj, target);
            }
        }

    }
}
