﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Diagnostics;
using System.Net;

namespace CallistoX
{
    public partial class PluginGUI : UserControl
    {
        public PluginGUI()
        {
            InitializeComponent();
        }

        public void SetValues()
        {
            Settings.CanSave = false;
            this.checkBox1.Checked = Settings.ChatLogOn;
            this.checkBox2.Checked = Settings.AgeOn;
            this.numericUpDown1.Value = Settings.Age;
            this.checkBox3.Checked = Settings.CbansOn;
            this.numericUpDown2.Value = Settings.CbansInterval;
            this.checkBox4.Checked = Settings.AllowMale;
            this.checkBox5.Checked = Settings.AllowFemale;
            this.checkBox6.Checked = Settings.AllowUnknown;
            Settings.CanSave = true;
            CmdLevelManager.SetValues(this.dataGridView1);
            StringTemplate.Load(false);
            StringTemplate.Populate(this.dataGridView2);

            if (DefaultAvatar.LoadDefaultAvatar())
                this.pictureBox2.Image = DefaultAvatar.DemoImage;
            else
                this.pictureBox2.Image = DefaultAvatar.ClearedImage;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e) // chat logging
        {
            Settings.ChatLogOn = this.checkBox1.Checked;
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) // show chatlog
        {
            ChatLog.ShowLogs();
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e) // age ban
        {
            Settings.AgeOn = this.checkBox2.Checked;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e) // age change
        {
            Settings.Age = (int)this.numericUpDown1.Value;
        }

        private void checkBox4_CheckedChanged(object sender, EventArgs e) // male
        {
            Settings.AllowMale = this.checkBox4.Checked;
        }

        private void checkBox5_CheckedChanged(object sender, EventArgs e) // female
        {
            Settings.AllowFemale = this.checkBox5.Checked;
        }

        private void checkBox6_CheckedChanged(object sender, EventArgs e) // unknown
        {
            Settings.AllowUnknown = this.checkBox6.Checked;
        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e) // cbans
        {
            Settings.CbansOn = this.checkBox3.Checked;
        }

        private void numericUpDown2_ValueChanged(object sender, EventArgs e) // cbans change
        {
            Settings.CbansInterval = (int)this.numericUpDown2.Value;
        }

        private void linkLabel2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) // admin log
        {
            AdminLog.ShowLog();
        }

        private void linkLabel3_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) // edit motd
        {
            MotdManager.EditMOTD();
        }

        private void dataGridView1_EditingControlShowing(object sender, DataGridViewEditingControlShowingEventArgs e) // editing cmd level
        {
            ComboBox c = (ComboBox)e.Control;
            
            if (c != null)
            {
                c.SelectedIndexChanged -= new EventHandler(this.CmdLevelChanged);
                c.SelectedIndexChanged += new EventHandler(this.CmdLevelChanged);
            }
        }

        private void CmdLevelChanged(object sender, EventArgs e) // edited cmd level
        {
            ComboBox c = (ComboBox)sender;
            int row = this.dataGridView1.SelectedCells[0].RowIndex;
            String command = this.dataGridView1.Rows[row].Cells[0].Value.ToString().Substring(1);
            CmdLevelManager.UpdateItem(command, c.SelectedItem.ToString());
        }

        private void dataGridView2_SelectionChanged(object sender, EventArgs e)
        {
            if (this.dataGridView2.CurrentCell != null) // a little bit of help with highlighting
            {
                if (this.dataGridView2.CurrentCell.Tag == null)
                {
                    this.dataGridView2.DefaultCellStyle.SelectionBackColor = Color.Gainsboro;
                    this.dataGridView2.DefaultCellStyle.SelectionForeColor = Color.Maroon;
                }
                else
                {
                    this.dataGridView2.DefaultCellStyle.SelectionBackColor = Color.White;
                    this.dataGridView2.DefaultCellStyle.SelectionForeColor = Color.Black;
                }
            }
        }

        private void button1_Click(object sender, EventArgs e) // save and update string template
        {
            StringTemplate.SaveAndUpdate(this.dataGridView2);
        }

        private void linkLabel4_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                if (Callisto.Room.IsRunning)
                {
                    String str = Callisto.Hashlink.Encode(new ChannelObject
                    {
                        ExternalIP = IPAddress.Parse("127.0.0.1"),
                        LocalIP = IPAddress.Parse("127.0.0.1"),
                        Name = Callisto.Room.Name,
                        Topic = Callisto.Room.Name,
                        Port = Callisto.Room.Port
                    });

                    if (str != null)
                        Process.Start("arlnk://" + str);
                }
            }
            catch { }
        }

        private void button2_Click(object sender, EventArgs e) // add default avatar
        {
            OpenFileDialog f = new OpenFileDialog();
            f.Multiselect = false;
            f.Filter = "Supported file types (*.bmp;*.jpg;*.png)|*.bmp;*.jpg;*.png|Bitmap files (*.bmp)|*.bmp|Jpeg files (*.jpg)|*.jpg|PNG files (*.png)|*.png";

            if (f.ShowDialog() == DialogResult.OK)
                if (DefaultAvatar.UpdateDefaultAvatar(f.FileName))
                    this.pictureBox2.Image = DefaultAvatar.DemoImage;
        }

        private void button3_Click(object sender, EventArgs e) // clear default avatar
        {
            DefaultAvatar.ClearDefaultAvatar();
            this.pictureBox2.Image = DefaultAvatar.ClearedImage;
        }

        private void linkLabel5_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) // reset template
        {
            DialogResult result = MessageBox.Show("Are you sure you wish to reset your string template to the default values?",
                "CallistoX", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (result == DialogResult.Yes)
            {
                StringTemplate.Load(true);
                StringTemplate.Populate(this.dataGridView2);
            }
        }

        private void linkLabel6_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            DialogResult result = MessageBox.Show("Are you sure you wish to reload your string template from file?",
                "CallistoX", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (result == DialogResult.Yes)
            {
                StringTemplate.Load(false);
                StringTemplate.Populate(this.dataGridView2);
            }
        }


    }
}
