﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Xml.Linq;
using ICallisto;

namespace CallistoX
{
    class RoomSearch
    {
        private Thread thread;
        private List<RoomSearchObject> items = new List<RoomSearchObject>();

        public bool terminate = false;

        public int DBSize
        {
            get { return this.items.Count; }
        }

        public RoomSearch()
        {
            this.thread = new Thread(new ThreadStart(this.BackgroundWorker));
            this.thread.Start();
        }

        public void Kill()
        {
            this.terminate = true;
        }

        public void Query(ushort vroom, String query)
        {
            if (this.items.Count < 2)
            {
                Callisto.Print(vroom, "Callisto is building a channel list database for the first time, please try again in a few minutes");
                return;
            }

            String q = query.ToUpper();

            List<RoomSearchObject> qlist = items.FindAll(delegate(RoomSearchObject r)
            {
                return r.name.ToUpper().Contains(q) || r.topic.ToUpper().Contains(q);
            });

            if (qlist.Count > 1)
                qlist.Sort((x, y) =>
                {
                    if (x.usercount == y.usercount)
                        return 0;

                    return x.usercount > y.usercount ? -1 : 1;
                });

            if (Callisto.Room.Name.ToUpper().Contains(q) ||
                Callisto.Room.Topic.ToUpper().Contains(q))
            {
                qlist.RemoveAll(x => x.name == Callisto.Room.Name);
                RoomSearchObject mine = new RoomSearchObject
                {
                    hashlink = "arlnk://" + Callisto.Hashlink.Encode(new ChannelObject
                    {
                        ExternalIP = Callisto.Room.ExternalIP,
                        LocalIP = Callisto.Room.ExternalIP,
                        Name = Callisto.Room.Name,
                        Port = Callisto.Room.Port,
                        Topic = Callisto.Room.Topic
                    }),
                    ip = Callisto.Room.ExternalIP,
                    language = this.LanguageCodeToString((byte)Callisto.Room.Language),
                    name = Callisto.Room.Name,
                    port = Callisto.Room.Port,
                    server = "Callisto",
                    topic = Callisto.Room.Topic,
                    usercount = (ushort)Callisto.Stats.CurrentUserCount
                };
                qlist.Insert(0, mine);
            }

            if (qlist.Count == 0)
            {
                if (StringTemplate.GetString(STType.Roomsearch, 0) != null)
                    Callisto.Print(vroom, StringTemplate.GetString(STType.Roomsearch, 0).Replace("+x", query), vroom == 0);
            }
            else
            {
                if (StringTemplate.GetString(STType.Roomsearch, 1) != null)
                    Callisto.Print(vroom, StringTemplate.GetString(STType.Roomsearch, 1).Replace("+x", query), vroom == 0);

                int count = 0;

                foreach (RoomSearchObject r in qlist)
                {
                    Callisto.Print(vroom, "");

                    for (int i = 2; i < 8; i++)
                        if (StringTemplate.GetString(STType.Roomsearch, i) != null)
                            Callisto.Print(vroom, this.FormatString(StringTemplate.GetString(STType.Roomsearch, i), r), vroom == 0);

                    if (count++ > 3)
                        break;
                }
            }
        }

        private String FormatString(String str, RoomSearchObject r)
        {
            return str.Replace("+n", r.name)
                .Replace("+t", r.topic)
                .Replace("+l", r.language)
                .Replace("+s", r.server)
                .Replace("+u", r.usercount.ToString())
                .Replace("+h", r.hashlink);
        }

        private void BackgroundWorker()
        {
            List<RoomSearchObject> servers = new List<RoomSearchObject>();
            this.terminate = false;
            this.LoadXml(ref servers);

            if (servers.Count == 0)
                this.LoadDefault(ref servers);
            else this.items = new List<RoomSearchObject>(servers);

            uint last_list_grab = (Helpers.UnixTimestamp() - 1800);

            while (true)
            {
                if (terminate)
                    return;

                uint now = Helpers.UnixTimestamp();

                if ((last_list_grab + 1800) < now) // wake up
                {
                    if (items.Count > 0)
                        servers = new List<RoomSearchObject>(items.ToArray());

                    List<RoomSearchObject> accepted = new List<RoomSearchObject>();
                    Socket usock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                    usock.Blocking = false;
                    int pos = 0;
                    int timeout = 0;
                    uint last_received = now;

                    while (true)
                    {
                        uint time = Helpers.UnixTimestamp();

                        if (terminate)
                            return;

                        int to_now = (int)((last_received + 10) - time);

                        if (to_now != timeout)
                            timeout = to_now;

                        if ((last_received + 10) < time || to_now == 0)
                            break;

                        if (pos < servers.Count)
                        {
                            try
                            {
                                usock.SendTo(new byte[] { 2 }, (EndPoint)new IPEndPoint(servers[pos].ip, servers[pos].port));
                            }
                            catch { }
                            pos++;
                            last_received = time;
                        }

                        Thread.Sleep(50);

                        if (usock.Available > 0)
                        {
                            EndPoint ep = (EndPoint)new IPEndPoint(IPAddress.Any, 0);
                            byte[] buf1 = new byte[1024];
                            int size = 0;

                            try
                            {
                                size = usock.ReceiveFrom(buf1, ref ep);
                            }
                            catch { }

                            if (size > 0)
                            {
                                try
                                {
                                    UdpPacketReader packet = new UdpPacketReader(new List<byte>(buf1).GetRange(0, size).ToArray());

                                    if (packet.ReadByte() == 3)
                                    {
                                        RoomSearchObject cobj = new RoomSearchObject();
                                        cobj.ip = ((IPEndPoint)ep).Address;
                                        cobj.port = packet.ReadInt16();
                                        cobj.usercount = packet.ReadInt16();
                                        cobj.name = packet.ReadString();
                                        cobj.topic = packet.ReadString();
                                        cobj.language = this.LanguageCodeToString(packet.ReadByte());
                                        cobj.server = this.ExtractServerName(packet.ReadString());
                                        packet.ReadByte();

                                        while (packet.Remaining() >= 6)
                                        {
                                            RoomSearchObject _server = new RoomSearchObject();
                                            _server.ip = packet.ReadIP();
                                            _server.port = packet.ReadInt16();

                                            if (servers.Find(x => x.ip.Equals(_server.ip)) == null)
                                                servers.Add(_server);
                                        }

                                        last_received = time;
                                        cobj.hashlink = "arlnk://" + Callisto.Hashlink.Encode(new ChannelObject
                                        {
                                            ExternalIP = cobj.ip,
                                            LocalIP = cobj.ip,
                                            Name = cobj.name,
                                            Port = cobj.port,
                                            Topic = String.Empty
                                        });

                                        if (cobj.usercount < 200 || (cobj.server == "sb0t" || cobj.server == "Callisto"))
                                            accepted.Add(cobj); // we know these servers don't have fake user counts
                                    }
                                }
                                catch { }
                            }
                        }

                        Thread.Sleep(50);
                    }

                    if (accepted.Count > 2)
                    {
                        this.items = new List<RoomSearchObject>(accepted.ToArray());
                        this.SaveXml();
                    }

                    last_list_grab = now;
                }

                Thread.Sleep(50);
            }
        }

        private String ExtractServerName(String version)
        {
            if (version.Contains("sb0t"))
                return "sb0t";

            if (version.Contains("Arca Eclipse"))
                return "AE";

            if (version.ToUpper().Contains("CALLISTO"))
                return "Callisto";

            if (version.ToUpper().Contains("SURFICTION"))
                return "Surf";

            if (version.ToUpper().Contains("AURA"))
                return "AurA";

            return "Ares";
        }

        private String LanguageCodeToString(byte code)
        {
            switch (code)
            {
                case 11:
                    return "Arabic";
                case 12:
                    return "Chinese";
                case 14:
                    return "Czech";
                case 15:
                    return "Danish";
                case 16:
                    return "Dutch";
                case 10:
                    return "English";
                case 27:
                    return "Finnish";
                case 28:
                    return "French";
                case 29:
                    return "German";
                case 30:
                    return "Italian";
                case 17:
                    return "Japanese";
                case 19:
                    return "Kirghiz";
                case 20:
                    return "Polish";
                case 21:
                    return "Portuguese";
                case 31:
                    return "Russian";
                case 22:
                    return "Slovak";
                case 23:
                    return "Spanish";
                case 25:
                    return "Swedish";
                case 26:
                    return "Turkish";
                default:
                    return "English";
            }
        }

        private void SaveXml()
        {
            XDocument xml = new XDocument(new XElement("Roomsearch"));

            foreach (RoomSearchObject i in (this.items.Count > 750 ? this.items.GetRange(0, 750) : this.items))
            {
                xml.Element("Roomsearch").Add(new XElement("Record",
                    new XElement("name", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.name))),
                    new XElement("topic", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.topic))),
                    new XElement("server", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.server))),
                    new XElement("hashlink", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.hashlink))),
                    new XElement("language", Convert.ToBase64String(Encoding.UTF8.GetBytes(i.language.ToString()))),
                    new XElement("usercount", i.usercount.ToString()),
                    new XElement("ip", Convert.ToBase64String(i.ip.GetAddressBytes())),
                    new XElement("port", i.port.ToString())));
            }

            try
            {
                xml.Save(Callisto.DataPath + "roomsearch.xml");
            }
            catch { }
        }

        private void LoadXml(ref List<RoomSearchObject> loadxml)
        {
            try
            {
                XDocument xml = XDocument.Load(Callisto.DataPath + "roomsearch.xml");

                loadxml = (from i in xml.Descendants("Record")
                              select new RoomSearchObject
                                  {
                                      name = Encoding.UTF8.GetString(Convert.FromBase64String(i.Element("name").Value)),
                                      topic = Encoding.UTF8.GetString(Convert.FromBase64String(i.Element("topic").Value)),
                                      server = Encoding.UTF8.GetString(Convert.FromBase64String(i.Element("server").Value)),
                                      hashlink = Encoding.UTF8.GetString(Convert.FromBase64String(i.Element("hashlink").Value)),
                                      language = Encoding.UTF8.GetString(Convert.FromBase64String(i.Element("language").Value)),
                                      usercount = ushort.Parse(i.Element("usercount").Value),
                                      ip = new IPAddress(Convert.FromBase64String(i.Element("ip").Value)),
                                      port = ushort.Parse(i.Element("port").Value)
                                  }).ToList<RoomSearchObject>();
            }
            catch { loadxml = new List<RoomSearchObject>(); }
        }

        private void LoadDefault(ref List<RoomSearchObject> loaddefault)
        {
            loaddefault = new List<RoomSearchObject>();

            if (Callisto.ServerNodes != null)
            {
                foreach (IPEndPoint addr in Callisto.ServerNodes)
                    loaddefault.Add(new RoomSearchObject
                    {
                        ip = addr.Address,
                        port = (ushort)addr.Port
                    });
            }
        }
    }

    class RoomSearchObject
    {
        public String name;
        public String topic;
        public String server;
        public String hashlink;
        public String language;
        public ushort usercount;
        public IPAddress ip;
        public ushort port;
    }

    class ChannelObject : IChannelObject
    {
        public ushort Port { get; set; }
        public IPAddress LocalIP { get; set; }
        public IPAddress ExternalIP { get; set; }
        public String Name { get; set; }
        public String Topic { get; set; }
    }

    class UdpPacketReader
    {
        private int Position = 0;
        private List<byte> Data = new List<byte>();

        public UdpPacketReader(byte[] bytes)
        {
            this.Data.Clear();
            this.Position = 0;
            this.Data.AddRange(bytes);
        }

        public int Remaining()
        {
            return this.Data.Count - this.Position;
        }

        public byte ReadByte()
        {
            byte tmp = this.Data[this.Position];
            this.Position++;
            return tmp;
        }

        public ushort ReadInt16()
        {
            ushort tmp = BitConverter.ToUInt16(this.Data.ToArray(), this.Position);
            this.Position += 2;
            return tmp;
        }

        public IPAddress ReadIP()
        {
            byte[] tmp = new byte[4];
            Array.Copy(this.Data.ToArray(), this.Position, tmp, 0, tmp.Length);
            this.Position += 4;
            return new IPAddress(tmp);
        }

        public String ReadString()
        {
            ushort len = BitConverter.ToUInt16(this.Data.ToArray(), this.Position);
            this.Position += 2;
            String str = Encoding.UTF8.GetString(this.Data.ToArray(), this.Position, len);
            this.Position += len;
            return str;
        }
    }
}
