﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class Topics
    {
        private static List<TopicItem> items = new List<TopicItem>();
        private static int clock_tick = 0;

        public static void UpdateClock()
        {
            if (Settings.TopicClock)
            {
                if (clock_tick++ > 30)
                {
                    clock_tick = 0;
                    DateTime now = DateTime.Now;
                    int h = now.Hour;
                    int m = now.Minute;
                    bool pm = h >= 12;

                    if (h == 0)
                        h = 12;
                    else if (h > 12)
                        h -= 12;

                    String clock = h + ":" + (m < 10 ? ("0" + m) : m.ToString()) + " " + (pm ? "PM" : "AM");
                    String str = StringTemplate.GetString(STType.ClockTopic, 0);

                    if (str != null)
                    {
                        str = str.Replace("+c", clock).Replace("+t", Callisto.Room.Topic);

                        for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                            if (u.Vroom == 0)
                                u.SetTopic(str);
                    }
                }
            }
        }

        public static void ForceClock(bool show)
        {
            DateTime now = DateTime.Now;
            int h = now.Hour;
            int m = now.Minute;
            bool pm = h >= 12;

            if (h == 0)
                h = 12;
            else if (h > 12)
                h -= 12;

            String clock = h + ":" + (m < 10 ? ("0" + m) : m.ToString()) + " " + (pm ? "PM" : "AM");
            String str = StringTemplate.GetString(STType.ClockTopic, 0);

            if (str != null)
            {
                str = str.Replace("+c", clock).Replace("+t", Callisto.Room.Topic);

                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                    if (u.Vroom == 0)
                        u.SetTopic(show ? str : Callisto.Room.Topic);
            }
        }

        public static void ForceClock(IUser userobj)
        {
            DateTime now = DateTime.Now;
            int h = now.Hour;
            int m = now.Minute;
            bool pm = h >= 12;

            if (h == 0)
                h = 12;
            else if (h > 12)
                h -= 12;

            String clock = h + ":" + (m < 10 ? ("0" + m) : m.ToString()) + " " + (pm ? "PM" : "AM");
            String str = StringTemplate.GetString(STType.ClockTopic, 0);

            if (str != null)
            {
                str = str.Replace("+c", clock).Replace("+t", Callisto.Room.Topic);
                userobj.SetTopic(str);
            }
        }

        public static void SetTopic(IUser userobj, String text)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : userobj.Name;

            if (userobj.Vroom == 0)
            {
                Callisto.Room.Topic = text;

                if (StringTemplate.GetString(STType.AdminCommands, 20) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 20).Replace("+n", _admin));

                AdminLog.AddLine(userobj.Name + " updated main topic");
            }
            else // vroom
            {
                if (items.Find(x => x.room == userobj.Vroom) != null)
                    items.Find(x => x.room == userobj.Vroom).text = text;
                else
                    items.Add(new TopicItem(text, userobj.Vroom));

                for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                    if (u.Vroom == userobj.Vroom)
                        u.SetTopic(text);

                Save();

                if (StringTemplate.GetString(STType.AdminCommands, 20) != null)
                    Callisto.Print(StringTemplate.GetString(STType.AdminCommands, 21).Replace("+n", _admin).Replace("+x", userobj.Vroom.ToString()));

                AdminLog.AddLine(userobj.Name + " updated vroom " + userobj.Vroom + " topic");
            }
        }

        public static void GetTopic(IUser userobj)
        {
            if (items.Find(x => x.room == userobj.Vroom) != null)
                userobj.SetTopic(items.Find(x => x.room == userobj.Vroom).text);
            else if (Settings.TopicClock)
                ForceClock(userobj);
            else
                userobj.SetTopic(Callisto.Room.Topic);
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "topics.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("topics");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("text");
                    String text = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("room");
                    ushort room = (ushort)xml.ReadElementContentAsInt();
                    items.Add(new TopicItem(text, room));
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "topics.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("topics");

                foreach (TopicItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("text", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.text)));
                    xml.WriteElementString("room", a.room.ToString());
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class TopicItem
    {
        public String text;
        public ushort room;

        public TopicItem(String text, ushort room)
        {
            this.text = text;
            this.room = room;
        }
    }
}
