﻿using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Text;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class Trace
    {
        private Thread thread;
        private bool busy = false;

        private ushort vroom = 0;
        private String query = null;
        private String name = null;

        public void Query(ushort vroom, IUser query)
        {
            if (busy)
            {
                if (StringTemplate.GetString(STType.Trace, 0) != null)
                    Callisto.Print(vroom, StringTemplate.GetString(STType.Trace, 0), vroom == 0);
                return;
            }

            this.vroom = vroom;
            this.query = query.ExternalIP.ToString();
            this.name = query.Name;
            this.thread = new Thread(new ThreadStart(this.Work));
            this.thread.Start();
        }

        public void QueryByIP(ushort vroom, String ip)
        {
            if (busy)
            {
                if (StringTemplate.GetString(STType.Trace, 0) != null)
                    Callisto.Print(vroom, StringTemplate.GetString(STType.Trace, 0), vroom == 0);
                return;
            }

            IPAddress test;

            if (!IPAddress.TryParse(ip, out test))
                return;

            this.vroom = vroom;
            this.query = ip;
            this.name = ip;
            this.thread = new Thread(new ThreadStart(this.Work));
            this.thread.Start();
        }

        private void Work()
        {
            this.busy = true;

            try
            {
                WebRequest request = WebRequest.Create("http://chatrooms.marsproject.net/geo/?q=" + this.query);
                WebResponse response = request.GetResponse();
                Stream stream = response.GetResponseStream();
                StreamReader reader = new StreamReader(stream, Encoding.Default);
                String[] lines = reader.ReadToEnd().Split(new String[] { "\n" }, StringSplitOptions.None);

                TraceObject obj = new TraceObject();
                obj.country = lines[0];
                obj.region = lines[1];
                obj.city = lines[2];
                obj.SetTime(lines[3], lines[4]);  // la lo

                reader.Close();
                stream.Flush();
                stream.Close();
                response.Close();

                if (StringTemplate.GetString(STType.Trace, 2) != null)
                {
                    WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 2).Replace("+x", this.name), this.vroom == 0));
                    WebResults.Results.Add(new WebResult(this.vroom, "", this.vroom == 0));
                }

                if (obj.country.Length > 0)
                    if (StringTemplate.GetString(STType.Trace, 3) != null)
                        WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 3).Replace("+x", obj.country), this.vroom == 0));

                if (obj.region.Length > 0)
                    if (StringTemplate.GetString(STType.Trace, 4) != null)
                        WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 4).Replace("+x", obj.region), this.vroom == 0));

                if (obj.city.Length > 0)
                    if (StringTemplate.GetString(STType.Trace, 5) != null)
                        WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 5).Replace("+x", obj.city), this.vroom == 0));

                if (obj.got_time)
                    if (StringTemplate.GetString(STType.Trace, 6) != null)
                        WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 6).Replace("+x", obj.localtime), this.vroom == 0));

                if (StringTemplate.GetString(STType.Trace, 7) != null)
                {
                    WebResults.Results.Add(new WebResult(this.vroom, "", this.vroom == 0));
                    WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 7), this.vroom == 0));
                }
            }
            catch
            {
                if (StringTemplate.GetString(STType.Trace, 1) != null)
                    WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.Trace, 1).Replace("+x", this.name), this.vroom == 0));
            }

            this.busy = false;
        }
    }

    class TraceObject
    {
        public String city = null;
        public String region = null;
        public String country = null;
        public int offset = 0;
        public bool dst = false;
        public bool got_time = false;

        public void SetTime(String _lat, String _lon)
        {
            float lat, lon;

            if (!float.TryParse(_lat, out lat))
                return;

            if (!float.TryParse(_lon, out lon))
                return;

            got_time = true;

            DateTime d = DateTime.UtcNow;

            if (lat > 15) // northern hemisphere
                if (d.Month > 3 && d.Month < 11)
                    this.dst = true;

            if (lat < -15) // southern hemisphere
                if (d.Month > 9 || d.Month < 4)
                    this.dst = true;

            this.offset = (int)(lon / 15);

            if (lon > 0)
                this.offset += 1;

            this.offset *= 3600; // to seconds
        }

        public String localtime
        {
            get
            {
                DateTime d = DateTime.UtcNow.AddSeconds(offset);

                if (this.dst)
                    d = d.AddHours(1);

                int h = d.Hour;
                int m = d.Minute;
                bool am = true;

                if (h > 11)
                    am = !am;

                if (h > 12)
                    h -= 12;

                if (h == 0)
                    h += 12;

                return h + ":" + (m < 10 ? ("0" + m) : m.ToString()) + " " + (am ? "AM" : "PM");
            }
        }
    }
}
