﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class URLManager
    {
        private static List<URLItem> items = new List<URLItem>();
        private static int Tick = 0;
        private static int Position = 0;
        private static URLItem Current = new URLItem { address = String.Empty, text = String.Empty };

        public static void ListURLs(IUser userobj)
        {
            for (int i = 0; i < items.Count; i++)
                Callisto.Print(userobj, i + " - " + items[i].address + " " + items[i].text);
        }

        public static void AddItem(IUser admin, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            int i = args.IndexOf(" ");

            if (i > -1)
            {
                String a = args.Substring(0, i).Trim();
                String t = args.Substring(i + 1).Trim();

                if (!String.IsNullOrEmpty(a) && !String.IsNullOrEmpty(t))
                {
                    items.Add(new URLItem { address = a, text = t });
                    Save();

                    if (StringTemplate.GetString(STType.URL, 0) != null)
                        Callisto.Print(StringTemplate.GetString(STType.URL, 0).Replace("+n", _admin), true);

                    AdminLog.AddLine("url tag added by " + admin.Name);
                    
                    if (items.Count == 1)
                    {
                        Tick = 11;
                        Update();
                    }
                }
            }
        }

        public static void RemoveItem(IUser admin, int ident)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (ident < 0 || ident >= items.Count)
                return;

            items.RemoveAt(ident);
            Save();

            if (StringTemplate.GetString(STType.URL, 1) != null)
                Callisto.Print(StringTemplate.GetString(STType.URL, 1).Replace("+n", _admin), true);

            AdminLog.AddLine("url tag removed by " + admin.Name);

            if (items.Count > 0)
            {
                Tick = 11;
                Update();
            }
            else
            {
                Callisto.Room.UpdateURL();
                Current = new URLItem { address = String.Empty, text = String.Empty };
            }
        }

        public static void URLOn(IUser admin, String args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;

            if (args == "on" && !Settings.UrlOn)
            {
                if (StringTemplate.GetString(STType.EnableDisable, 32) != null)
                    Callisto.Print(StringTemplate.GetString(STType.EnableDisable, 32).Replace("+n", _admin), true);

                AdminLog.AddLine("url tags enabled by " + admin.Name);
                Settings.UrlOn = true;

                if (items.Count > 0)
                {
                    Tick = 11;
                    Update();
                }
                else
                {
                    Callisto.Room.UpdateURL();
                    Current = new URLItem { address = String.Empty, text = String.Empty };
                }
            }
            
            if (args == "off" && Settings.UrlOn)
            {
                if (StringTemplate.GetString(STType.EnableDisable, 33) != null)
                    Callisto.Print(StringTemplate.GetString(STType.EnableDisable, 33).Replace("+n", _admin), true);

                AdminLog.AddLine("url tags disabled by " + admin.Name);
                Settings.UrlOn = false;
                Callisto.Room.UpdateURL();
                Current = new URLItem { address = String.Empty, text = String.Empty };
            }
        }

        public static void Update()
        {
            if (!Settings.UrlOn || items.Count == 0)
                return;

            if (Tick++ > 10)
            {
                Tick = 0;
                Position++;

                if (Position >= items.Count)
                    Position = 0;

                URLItem tmp = items[Position];

                if (tmp.address != Current.address || tmp.text != Current.text)
                {
                    Callisto.Room.UpdateURL(items[Position].address, items[Position].text);
                    Current = tmp;
                }
            }
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "urls.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("urls");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("address");
                    String a = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("text");
                    String t = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    
                    items.Add(new URLItem
                    {
                        address = a,
                        text = t
                    });
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }

            if (items.Count == 0)
                Callisto.Room.UpdateURL();
            else
                Callisto.Room.UpdateURL(items[0].address, items[0].text);
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "urls.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("urls");

                foreach (URLItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("address", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.address)));
                    xml.WriteElementString("text", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.text)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }

    class URLItem
    {
        public String address { get; set; }
        public String text { get; set; }
    }
}
