﻿using System;
using System.IO;
using System.Net;
using System.Threading;
using System.Web;
using System.Text;
using System.Text.RegularExpressions;
using System.Runtime.Serialization.Json;

namespace CallistoX
{
    class UrbanDictionary
    {
        private Thread thread;
        private bool busy = false;

        private ushort vroom = 0;
        private String query = String.Empty;

        public void Query(ushort vroom, String query)
        {
            if (busy)
            {
                if (StringTemplate.GetString(STType.UrbanDictionary, 0) != null)
                    Callisto.Print(this.vroom, StringTemplate.GetString(STType.UrbanDictionary, 0), this.vroom == 0);
                return;
            }

            this.vroom = vroom;
            this.query = query;
            this.thread = new Thread(new ThreadStart(this.Work));
            this.thread.Start();
        }

        private void Work()
        {
            this.busy = true;

            try
            {
                UrbanItem[] results = GetDefinitions(this.query);

                if (results.Length > 0)
                {
                    for (int x = 0; x < results.Length; x++)
                    {
                        if (x > 1)
                            break; // max 2 definitions per query!

                        String[] ud_split = results[x].Definition.Split(new String[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

                        if (x > 0) WebResults.Results.Add(new WebResult(this.vroom, ""));

                        if (StringTemplate.GetString(STType.UrbanDictionary, 2) != null)
                        {
                            WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.UrbanDictionary, 2).Replace("+x", this.query), this.vroom == 0));
                            WebResults.Results.Add(new WebResult(this.vroom, "", this.vroom == 0));
                        }

                        for (int i = 0; i < ud_split.Length; i++)
                        {
                            if (ud_split[i].Length > 1200) ud_split[i] = ud_split[i].Substring(0, 1200);

                            if (StringTemplate.GetString(STType.UrbanDictionary, 3) != null)
                                WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.UrbanDictionary, 3).Replace("+x", ud_split[i]), this.vroom == 0));

                            if (i > 50)
                                break; // max 50 lines per definition
                        }

                        ud_split = results[x].Example.Split(new String[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

                        if (ud_split.Length > 0)
                        {
                            if (StringTemplate.GetString(STType.UrbanDictionary, 4) != null)
                            {
                                WebResults.Results.Add(new WebResult(this.vroom, "", this.vroom == 0));
                                WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.UrbanDictionary, 4).Replace("+x", this.query), this.vroom == 0));
                                WebResults.Results.Add(new WebResult(this.vroom, "", this.vroom == 0));
                            }

                            for (int i = 0; i < ud_split.Length; i++)
                            {
                                if (ud_split[i].Length > 1200) ud_split[i] = ud_split[i].Substring(0, 1200);

                                if (StringTemplate.GetString(STType.UrbanDictionary, 5) != null)
                                    WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.UrbanDictionary, 5).Replace("+x", ud_split[i]), this.vroom == 0));

                                if (i > 50)
                                    break; // max 50 lines per example
                            }
                        }
                    }
                }
                else
                {
                    if (StringTemplate.GetString(STType.UrbanDictionary, 1) != null)
                        WebResults.Results.Add(new WebResult(this.vroom, StringTemplate.GetString(STType.UrbanDictionary, 1).Replace("+x", this.query), this.vroom == 0));
                }
            }
            catch { }

            this.busy = false;
        }

        private UrbanItem[] GetDefinitions(String _text)
        {
            try
            {
                WebRequest request = WebRequest.Create("http://www.urbandictionary.com/iphone/search/define?term=" + Uri.EscapeDataString(_text));
                WebResponse response = request.GetResponse();
                Stream stream = response.GetResponseStream();
                StreamReader reader = new StreamReader(stream, Encoding.UTF8);
                String json = reader.ReadToEnd();
                reader.Close();
                stream.Flush();
                stream.Close();
                response.Close();
                UrbanResponse result = Deserialize(json);

                if (result == null)
                    return new UrbanItem[] { };

                return result.list;
            }
            catch { return new UrbanItem[] { }; }
        }

        private UrbanResponse Deserialize(String json)
        {
            UrbanResponse obj = Activator.CreateInstance<UrbanResponse>();
            MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(json));
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(obj.GetType());

            try
            {
                obj = (UrbanResponse)serializer.ReadObject(ms);
            }
            catch
            {
                ms.Close();
                return null;
            }

            ms.Close();
            return obj;
        }
    }

    [Serializable]
    public class UrbanResponse
    {
        public String total;
        public String pages;
        public String result_type;
        public String has_related_words;
        public UrbanItem[] list;
    }

    [Serializable]
    public class UrbanItem
    {
        public String thumbs_up;
        public String defid;
        public String definition;
        public String thumbs_down;
        public String word;
        public String current_vote;
        public String example;
        public String permalink;

        public String Definition
        {
            get
            {
                String str = this.definition.Replace("\r", "").Replace("<br/>", "\r\n");

                while (str.Contains("\r\n\r\n"))
                    str = str.Replace("\r\n\r\n", "\r\n");  // max 1 line break at a time

                str = Regex.Replace(str, @"<(.|\n)*?>", String.Empty, RegexOptions.IgnoreCase);
                str = HttpUtility.HtmlDecode(str);
                return str;
            }
        }

        public String Example
        {
            get
            {
                String str = this.example.Replace("\r", "").Replace("<br/>", "\r\n");

                while (str.Contains("\r\n\r\n"))
                    str = str.Replace("\r\n\r\n", "\r\n");  // max 1 line break at a time

                str = Regex.Replace(str, @"<(.|\n)*?>", String.Empty, RegexOptions.IgnoreCase);
                str = HttpUtility.HtmlDecode(str);
                return str;
            }
        }
    }
}
//192