﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class Vrooms
    {
        private static List<ushort> items = new List<ushort>();

        public static void Locate(IUser userobj)
        {
            String str1 = StringTemplate.GetString(STType.Vrooms, 0);
            String str2 = StringTemplate.GetString(STType.Vrooms, 1);

            for (IUser u = Callisto.FirstUser; u != null; u = u.NextUser)
                if (u.Vroom > 0)
                    if (items.Contains(u.Vroom) && str2 != null)
                        Callisto.Print(userobj, str2.Replace("+n", u.Name).Replace("+v", u.Vroom.ToString()));
                    else if (!items.Contains(u.Vroom) && str1 != null)
                        Callisto.Print(userobj, str1.Replace("+n", u.Name).Replace("+v", u.Vroom.ToString()));
        }

        public static void ProcessVroom(IUser userobj, String args)
        {
            switch (args)
            {
                case "lock":
                    LockVroom(userobj);
                    break;

                case "unlock":
                    UnlockVroom(userobj);
                    break;

                default:
                    ushort u;

                    if (ushort.TryParse(args, out u))
                        userobj.Vroom = u;

                    break;
            }
        }

        private static void LockVroom(IUser userobj)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : userobj.Name;

            if (userobj.Level != AdminLevel.Host)
                return;

            if (userobj.Vroom == 0)
                return;

            if (!items.Contains(userobj.Vroom))
            {
                items.Add(userobj.Vroom);
                Save();

                if (StringTemplate.GetString(STType.Vrooms, 2) != null)
                    Callisto.Print(StringTemplate.GetString(STType.Vrooms, 2).Replace("+n", _admin).Replace("+v", userobj.Vroom.ToString()));

                AdminLog.AddLine(userobj.Name + " has locked vroom " + userobj.Vroom);
            }
        }

        private static void UnlockVroom(IUser userobj)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : userobj.Name;

            if (userobj.Level != AdminLevel.Host)
                return;

            if (userobj.Vroom == 0)
                return;

            if (items.Contains(userobj.Vroom))
            {
                items.RemoveAll(x => x == userobj.Vroom);
                Save();

                if (StringTemplate.GetString(STType.Vrooms, 3) != null)
                    Callisto.Print(StringTemplate.GetString(STType.Vrooms, 3).Replace("+n", _admin).Replace("+v", userobj.Vroom.ToString()));

                AdminLog.AddLine(userobj.Name + " has unlocked vroom " + userobj.Vroom);
            }
        }

        public static bool CanJoin(IUser userobj, ushort vroom)
        {
            if (userobj.Level == AdminLevel.Host)
                return true;

            if (items.Contains(vroom))
            {
                if (StringTemplate.GetString(STType.Vrooms, 4) != null)
                    Callisto.Print(userobj, StringTemplate.GetString(STType.Vrooms, 4).Replace("+n", userobj.Name).Replace("+v", vroom.ToString()));
                return false;
            }

            return true;
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "vrooms.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("vrooms");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("room");
                    items.Add((ushort)xml.ReadElementContentAsInt());
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "vrooms.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("vrooms");

                foreach (ushort a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("room", a.ToString());
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }
    }
}
