﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.IO;
using System.Threading;
using System.Net;
using ICallisto;

namespace CallistoX
{
    class Weather
    {
        private Thread thread;
        private bool busy = false;

        private ushort vroom = 0;
        private String query = null;
        private String name = null;

        public void Query(ushort vroom, IUser query)
        {
            if (busy)
            {
                Callisto.Print(this.vroom, "wait a few seconds and then try again");
                return;
            }

            this.vroom = vroom;
            this.query = query.ExternalIP.ToString();
            this.name = query.Name;
            this.thread = new Thread(new ThreadStart(this.Work));
            this.thread.Start();
        }

        private void Work()
        {
            this.busy = true;

            try
            {
                WebRequest request = WebRequest.Create("http://chatrooms.marsproject.net/geo/?q=" + this.query);
                WebResponse response = request.GetResponse();
                Stream stream = response.GetResponseStream();
                StreamReader reader = new StreamReader(stream, Encoding.Default);
                String[] lines = reader.ReadToEnd().Split(new String[] { "\n" }, StringSplitOptions.None);

                String country = lines[0];
                String region = lines[1];
                String city = lines[2];

                reader.Close();
                stream.Flush();
                stream.Close();
                response.Close();

                WeatherResult result;
                List<String> args = new List<String>();

                if (!String.IsNullOrWhiteSpace(city))
                    args.Add(city);

                if (!String.IsNullOrWhiteSpace(region))
                    args.Add(region);

                if (!String.IsNullOrWhiteSpace(country))
                    args.Add(country);

                if (args.Count == 0)
                {
                    WebResults.Results.Add(new WebResult(this.vroom, "unable to detect weather for " + this.name));
                }
                else
                {
                    while (args.Count > 0)
                    {
                        result = this.GetResult(args.ToArray());

                        if (result != null)
                        {
                            WebResults.Results.Add(new WebResult(this.vroom, "Weather results for " + this.name));
                            WebResults.Results.Add(new WebResult(this.vroom, " "));
                            WebResults.Results.Add(new WebResult(this.vroom, result.Condition));
                            WebResults.Results.Add(new WebResult(this.vroom, result.Temp));
                            WebResults.Results.Add(new WebResult(this.vroom, result.Humidity));
                            WebResults.Results.Add(new WebResult(this.vroom, result.Wind));
                            break;
                        }
                        else args.RemoveAt(0);
                    }

                    if (args.Count == 0)
                    {
                        WebResults.Results.Add(new WebResult(this.vroom, "unable to detect weather for " + this.name));
                    }
                }
            }
            catch
            {
                WebResults.Results.Add(new WebResult(this.vroom, "unable to detect weather for " + this.name));
            }

            this.busy = false;
        }

        private WeatherResult GetResult(String[] args)
        {
            String url = "http://www.google.com/ig/api?weather=" + (String.Join(",", args).Replace(" ", "+"));
            XDocument xml = XDocument.Load(url);
            IEnumerable<XElement> weather = xml.Descendants("weather");
            IEnumerable<XElement> current = weather.Descendants("current_conditions");

            if (current.Count() == 0)
            {
                return null;
            }
            else
            {
                WeatherResult result = new WeatherResult();
                XElement now = current.ToArray()[0];
                XAttribute condition = now.Element("condition").Attributes().FirstOrDefault(x => x.Name == "data");
                XAttribute temp_f = now.Element("temp_f").Attributes().FirstOrDefault(x => x.Name == "data");
                XAttribute temp_c = now.Element("temp_c").Attributes().FirstOrDefault(x => x.Name == "data");
                XAttribute humidity = now.Element("humidity").Attributes().FirstOrDefault(x => x.Name == "data");
                XAttribute wind_condition = now.Element("wind_condition").Attributes().FirstOrDefault(x => x.Name == "data");

                if (condition != null)
                    result.Condition = "Condition: " + condition.Value;
                else
                    result.Condition = "Condition: unknown";

                if (temp_f != null)
                    result.TempF = temp_f.Value + "\u00B0F";
                else
                    result.TempF = null;

                if (temp_c != null)
                    result.TempC = temp_c.Value + "\u00B0C";
                else
                    result.TempC = null;

                if (result.TempC == null && result.TempF == null)
                    result.Temp = "Temperature: unknown";
                else if (result.TempC != null && result.TempF == null)
                    result.Temp = "Temperature: " + result.TempC;
                else if (result.TempC == null && result.TempF != null)
                    result.Temp = "Temperature: " + result.TempF;
                else
                    result.Temp = "Temperature: " + result.TempC + " | " + result.TempF;

                if (humidity != null)
                    result.Humidity = humidity.Value;
                else
                    result.Humidity = "Humidity: unknown";

                if (wind_condition != null)
                    result.Wind = wind_condition.Value;
                else
                    result.Wind = "Wind: unknown";

                return result;
            }
        }
    }

    class WeatherResult
    {
        public String Condition { get; set; }
        public String TempF { get; set; }
        public String TempC { get; set; }
        public String Humidity { get; set; }
        public String Wind { get; set; }
        public String Temp { get; set; }
    }
}
