﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using ICallisto;

namespace CallistoX
{
    class WhowasManager
    {
        private static List<WhowasItem> items = new List<WhowasItem>();
        private static int tick = 0;

        public static void Whowas(IUser admin, String args)
        {
            String query = args.Replace("\"", String.Empty).ToUpper(); // namr or ip
            List<WhowasItem> results = items.FindAll(x => x.Name.ToUpper().Contains(query) || x.ExternalIP.StartsWith(query));

            String str1 = StringTemplate.GetString(STType.Whowas, 0);
            String str2 = StringTemplate.GetString(STType.Whowas, 1);
            String str3 = args.Replace("\"", String.Empty);

            if (results.Count == 0)
            {
                if (str1 != null)
                    Callisto.Print(admin, str1.Replace("+q", str3));
            }
            else
            {
                if (str2 != null)
                    foreach (WhowasItem i in results)
                        Callisto.Print(admin, str2.Replace("+n", i.Name)
                            .Replace("+e", i.ExternalIP)
                            .Replace("+v", i.Version)
                            .Replace("+l", i.LocalIP)
                            .Replace("+p", i.Port.ToString())
                            .Replace("+t", Helpers.UnixTime2String(i.Time))
                            .Replace("+q", str3));
            }
        }
      

        public static void LastSeen(IUser userobj)
        {
            WhowasItem i = items.FindLast(x => x.ExternalIP == userobj.ExternalIP.ToString());

            if (i != null)
                if (i.Name != userobj.Name)
                    if (StringTemplate.GetString(STType.LastSeen, 0) != null)
                        Callisto.Print(StringTemplate.GetString(STType.LastSeen, 0).Replace("+n", userobj.Name).Replace("+x", i.Name).Replace("+ip", userobj.ExternalIP.ToString()));
        }

        public static void Load()
        {
            try
            {
                XDocument xml = XDocument.Load(Callisto.DataPath + "whowas.xml");

                items = (from i in xml.Descendants("Record")
                         select new WhowasItem
                             {
                                 Name = i.Element("Name").Value,
                                 Version = i.Element("Version").Value,
                                 ExternalIP = i.Element("ExternalIP").Value,
                                 LocalIP = i.Element("LocalIP").Value,
                                 Port = uint.Parse(i.Element("Port").Value),
                                 Time = uint.Parse(i.Element("Time").Value),
                             }).ToList<WhowasItem>();
            }
            catch { }
        }

        public static void Update()
        {
            if (tick++ < 300) // save file every 5 minutes
                return;

            tick = 0;

            if (items.Count > 5000)
                items.RemoveRange(0, 1000); // i have to concede a max number of records

            XDocument xml = new XDocument(new XElement("Whowas"));

            foreach (WhowasItem i in items)
            {
                xml.Element("Whowas").Add(new XElement("Record",
                    new XElement("Name", i.Name),
                    new XElement("Version", i.Version),
                    new XElement("ExternalIP", i.ExternalIP),
                    new XElement("LocalIP", i.LocalIP),
                    new XElement("Port", i.Port.ToString()),
                    new XElement("Time", i.Time.ToString())));
            }

            try
            {
                xml.Save(Callisto.DataPath + "whowas.xml");
            }
            catch { }
        }

        public static void AddRecord(IUser userobj)
        {
            items.RemoveAll(x => x.Name == userobj.Name && x.ExternalIP == userobj.ExternalIP.ToString());
            items.Add(new WhowasItem
            {
                Name = Helpers.StripColors(userobj.Name),
                Version = Helpers.StripColors(userobj.Version),
                ExternalIP = userobj.ExternalIP.ToString(),
                LocalIP = userobj.LocalIP.ToString(),
                Port = userobj.Port,
                Time = Helpers.UnixTimestamp()
            });
        }
    }

    class WhowasItem
    {
        public String Name { get; set; }
        public String Version { get; set; }
        public String ExternalIP { get; set; }
        public String LocalIP { get; set; }
        public uint Port { get; set; }
        public uint Time { get; set; }
    }
}
