﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Xml;
using ICallisto;

namespace CallistoX
{
    class WordFilter
    {
        private static String[] types = new String[]
        {
            "muzzle",
            "kiddy",
            "kill",
            "ban",
            "censor",
            "move",
            "redirect",
            "pm",
            "announce",
            "clone",
            "replace"
        };

        private static List<FilterItem> items = new List<FilterItem>();

        public static void AddFilter(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 2)
                return;

            String trigger = list[0];
            String type = list[1];
            String args = String.Empty;

            list.RemoveRange(0, 2);

            if (list.Count > 0)
                args = String.Join(", ", list.ToArray());

            if (types.Contains(type))
            {
                if (new List<String>(types).IndexOf(type) > 4)
                    if (args.Length == 0)
                        return; // need args

                int exists = items.FindIndex(x => x.trigger == trigger && x.type == type);

                if (exists > -1)
                    items[exists].args = args;
                else
                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });

                if (StringTemplate.GetString(STType.FilterContent, 6) != null)
                    Callisto.Print(StringTemplate.GetString(STType.FilterContent, 6).Replace("+n", _admin).Replace("+t", trigger).Replace("+x", type), true);

                AdminLog.AddLine("word filter added by " + admin.Name + " : " + trigger);
                Save();
            }
        }

        public static void ListFilters(IUser admin)
        {
            if (items.Count == 0)
            {
                if (StringTemplate.GetString(STType.FilterList, 0) != null)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 0));

                return;
            }

            if (StringTemplate.GetString(STType.FilterList, 1) != null)
                for (int i = 0; i < items.Count; i++)
                    Callisto.Print(admin, StringTemplate.GetString(STType.FilterList, 1).Replace("+id", i.ToString()).Replace("+t", items[i].trigger).Replace("+x", items[i].type));
        }

        public static void RemoveFilter(IUser admin, int ident)
        {
            if (ident < 0 || ident >= items.Count)
                return;
            
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            FilterItem item = items[ident];
            items.RemoveAt(ident);
            
            if (StringTemplate.GetString(STType.FilterContent, 7) != null)
                Callisto.Print(StringTemplate.GetString(STType.FilterContent, 7).Replace("+n", _admin).Replace("+t", item.trigger).Replace("+x", item.type), true);

            AdminLog.AddLine("word filter removed by " + admin.Name + " : " + item.trigger);
            Save();
        }

        public static void AddLine(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 2) // ident and text
                return;

            int ident;

            if (!int.TryParse(list[0], out ident))
                return;

            list.RemoveAt(0);
            String text = list.Count == 1 ? list[0] : String.Join(", ", list.ToArray());

            if (ident < 0 || ident >= items.Count)
                return;

            if (items[ident].type != "announce")
                return;

            items[ident].args += "\0" + text;

            if (StringTemplate.GetString(STType.FilterContent, 8) != null)
                Callisto.Print(StringTemplate.GetString(STType.FilterContent, 8).Replace("+n", _admin).Replace("+t", items[ident].trigger), true);

            Save();
        }

        public static void EditLine(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 3) // filter_id line_id text
                return;

            int ident1, ident2;

            if (!int.TryParse(list[0], out ident1) || !int.TryParse(list[1], out ident2))
                return;

            list.RemoveRange(0, 2);
            String text = list.Count == 1 ? list[0] : String.Join(", ", list.ToArray());

            if (ident1 < 0 || ident1 >= items.Count)
                return;

            if (items[ident1].type != "announce")
                return;

            list = new List<String>(items[ident1].args.Split(new String[] { "\0" }, StringSplitOptions.None));

            if (ident2 < 0 || ident2 >= list.Count)
                return;

            list[ident2] = text;
            items[ident1].args = String.Join("\0", list.ToArray());

            if (StringTemplate.GetString(STType.FilterContent, 9) != null)
                Callisto.Print(StringTemplate.GetString(STType.FilterContent, 9).Replace("+n", _admin).Replace("+t", items[ident1].trigger), true);

            Save();
        }

        public static void RemLine(IUser admin, String _args)
        {
            String _admin = Settings.StealthOn ? Callisto.Room.BotName : admin.Name;
            List<String> list = new List<String>(_args.Split(new String[] { ", " }, StringSplitOptions.None));

            if (list.Count < 2) // filter_id line_id
                return;

            int ident1, ident2;

            if (!int.TryParse(list[0], out ident1) || !int.TryParse(list[1], out ident2))
                return;

            if (ident1 < 0 || ident1 >= items.Count)
                return;

            if (items[ident1].type != "announce")
                return;

            list = new List<String>(items[ident1].args.Split(new String[] { "\0" }, StringSplitOptions.None));

            if (ident2 < 0 || ident2 >= list.Count)
                return;

            list.RemoveAt(ident2);

            if (list.Count > 0)
            {
                if (StringTemplate.GetString(STType.FilterContent, 10) != null)
                    Callisto.Print(StringTemplate.GetString(STType.FilterContent, 10).Replace("+n", _admin).Replace("+t", items[ident1].trigger), true);

                items[ident1].args = String.Join("\0", list.ToArray());
                Callisto.Print("line removed in filter [" + items[ident1].trigger + "] by " + _admin, true);
                Save();
            }
            else
            {
                FilterItem item = items[ident1];
                items.RemoveAt(ident1);

                if (StringTemplate.GetString(STType.FilterContent, 7) != null)
                    Callisto.Print(StringTemplate.GetString(STType.FilterContent, 7).Replace("+n", _admin).Replace("+t", item.trigger).Replace("+x", item.type), true);

                Save();
            }
        }

        public static void ViewFilter(IUser admin, String _args)
        {
            int ident;

            if (!int.TryParse(_args, out ident))
                return;

            if (ident < 0 || ident >= items.Count)
                return;

            if (items[ident].type != "announce")
                return;

            String[] lines = items[ident].args.Split(new String[] { "\0" }, StringSplitOptions.None);

            int count = 0;

            foreach (String str in lines)
                Callisto.Print(admin, (count++) + str.Replace("+n", admin.Name).Replace("+ip", admin.ExternalIP.ToString()));
        }

        public static void Load()
        {
            try
            {
                FileStream f = new FileStream(Callisto.DataPath + "wordfilters.xml", FileMode.Open);
                XmlReader xml = XmlReader.Create(new StreamReader(f));

                xml.MoveToContent();
                xml.ReadSubtree().ReadToFollowing("wordfilters");

                while (xml.ReadToFollowing("item"))
                {
                    xml.ReadSubtree().ReadToFollowing("trigger");
                    String trigger = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("type");
                    String type = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    xml.ReadToFollowing("args");
                    String args = Encoding.UTF8.GetString(Convert.FromBase64String(xml.ReadElementContentAsString()));
                    
                    items.Add(new FilterItem
                    {
                        trigger = trigger,
                        type = type,
                        args = args
                    });
                }

                xml.Close();
                f.Flush();
                f.Close();
            }
            catch { }
        }

        private static void Save()
        {
            try
            {
                XmlWriterSettings appearance = new XmlWriterSettings();
                appearance.Indent = true;
                XmlWriter xml = XmlWriter.Create(Callisto.DataPath + "wordfilters.xml", appearance);

                xml.WriteStartDocument();
                xml.WriteStartElement("wordfilters");

                foreach (FilterItem a in items)
                {
                    xml.WriteStartElement("item");
                    xml.WriteElementString("trigger", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.trigger)));
                    xml.WriteElementString("type", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.type)));
                    xml.WriteElementString("args", Convert.ToBase64String(Encoding.UTF8.GetBytes(a.args)));
                    xml.WriteEndElement();
                }

                xml.WriteEndElement();
                xml.WriteEndDocument();
                xml.Flush();
                xml.Close();
            }
            catch { }
        }

        public static bool FilterWhisper(String text)
        {
            String result = text;

            foreach (FilterItem i in items)
            {
                if (Regex.IsMatch(result, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                {
                    switch (i.type)
                    {
                        case "muzzle":
                        case "kiddy":
                        case "kill":
                        case "ban":
                        case "censor":
                        case "move":
                        case "redirect":
                        case "pm":
                        case "replace":
                            return false;
                    }
                }
            }

            return true;
        }

        public static String FilterBefore(IUser userobj, String text, bool emote)
        {
            String result = text;

            foreach (FilterItem i in items)
            {
                if (Regex.IsMatch(result, Regex.Escape(i.trigger), RegexOptions.IgnoreCase))
                {
                    switch (i.type)
                    {
                        case "muzzle":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                                MuzzleManager.AddFilterMuzzle(userobj);

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "kiddy":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                                KiddyManager.AddFilterKiddy(userobj);

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "kill":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                            {
                                if (StringTemplate.GetString(STType.FilterMessages, 7) != null)
                                    Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 7).Replace("+n", userobj.Name), true);

                                AdminLog.AddLine(userobj.Name + " killed by word filter");
                                userobj.Disconnect();
                            }

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "ban":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                            {
                                if (StringTemplate.GetString(STType.FilterMessages, 8) != null)
                                    Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 8).Replace("+n", userobj.Name), true);

                                BanManager.AddBanStatItem(new BanStatItem
                                {
                                    Name = userobj.Name,
                                    IP = userobj.ExternalIP,
                                    Admin = null,
                                    Index = 5
                                });
                                AdminLog.AddLine(userobj.Name + " banned by word filter");
                                userobj.Ban();
                            }

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "censor":
                            if (userobj.Level == AdminLevel.Regular)
                            {
                                if (StringTemplate.GetString(STType.FilterMessages, 10) != null && !userobj.IsLinked)
                                    Callisto.Print(userobj, StringTemplate.GetString(STType.FilterMessages, 10).Replace("+n", userobj.Name));
                                return String.Empty;
                            }
                            else break;

                        case "move":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                            {
                                ushort us;

                                if (ushort.TryParse(i.args, out us))
                                {
                                    if (StringTemplate.GetString(STType.FilterMessages, 9) != null)
                                        Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 9).Replace("+n", userobj.Name).Replace("+x", userobj.Vroom.ToString()), true);

                                    AdminLog.AddLine(userobj.Name + " moved to vroom " + userobj.Vroom + " by word filter");
                                    userobj.Vroom = us;
                                }
                            }

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "redirect":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                            {
                                IChannelObject chan = Callisto.Hashlink.Decode(i.args);

                                if (chan != null)
                                {
                                    if (StringTemplate.GetString(STType.FilterMessages, 11) != null)
                                        Callisto.Print(StringTemplate.GetString(STType.FilterMessages, 11).Replace("+n", userobj.Name).Replace("+x", chan.Name), true);

                                    AdminLog.AddLine(userobj.Name + " redirected to " + chan.Name + " by word filter");
                                    userobj.Redirect(i.args);
                                }
                            }

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "pm":
                            Callisto.SendPM(userobj, Callisto.Room.BotName, i.args.Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString()));
                            break;

                        case "announce":
                            if (!Settings.AdminAnnounceOn || userobj.Level > AdminLevel.Regular)
                            {
                                String[] lines = i.args.Split(new String[] { "\0" }, StringSplitOptions.None);
                                bool starts_with = result.ToUpper().StartsWith(i.trigger.ToUpper()) && result.Length > i.trigger.Length;
                                String plus_r = starts_with ? result.Substring(i.trigger.Length + 1) : String.Empty;

                                foreach (String str in lines)
                                    Callisto.Print(userobj.Vroom, str.Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString()).Replace("+r", plus_r), userobj.Vroom == 0);
                            }
                            
                            break;

                        case "clone":
                            if (!userobj.IsLinked && userobj.Level == AdminLevel.Regular)
                            {
                                if (emote)
                                    userobj.SendEmote(i.args.Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString()));
                                else userobj.SendText(i.args.Replace("+n", userobj.Name).Replace("+ip", userobj.ExternalIP.ToString()));
                            }

                            if (userobj.Level == AdminLevel.Regular)
                                return String.Empty;
                            else break;

                        case "replace":
                            if (userobj.Level == AdminLevel.Regular)
                                result = Regex.Replace(result, Regex.Escape(i.trigger), i.args, RegexOptions.IgnoreCase);
                            break;
                    }
                }
            }

            return result;
        }


    }

    class FilterItem
    {
        public String trigger { get; set; }
        public String type { get; set; }
        public String args { get; set; }
    }
}
